﻿using IQAP.Ofertas.Negocio.Datos;
using IQAP.Ofertas.Negocio.Modelo;
using IQAP.Shared.Negocio.API;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IQAP.Ofertas.Negocio.API
{
    public enum Precios
    {
        Ninguno = 0,
        Minimo = 1,
        Recomendado = 2,
        Coste = 3
    }
    
    public enum TiposValor
    {
        Ninguno = 0,
        ValorFinal = 1,
        Incremento = 2,
        Porcentaje = 3
    }

    public enum Criterios
    {
        ValoresCalculo = 1,
        Polimero = 2,
        Distribuidor = 3,
        AjusteEscaladoKg = 4,
        Area = 5,
        Comisionista = 6
    }

    public class CalculoAPI
    {
        public IEnumerable<Modelo.Criterio> ObtenerCriteriosPendientes(short idCalculo)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Calculos.SeleccionarCriteriosPendientes(idCalculo);
            }
        }

        public List<DatosCalculo> ObtenerParametrizaciones(short? idNegocio)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Calculos.SeleccionParametrizaciones(idNegocio);
            }
        }

        public void InsertarAplicacion(Aplicacion aplicacion)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                Repositorio.Aplicaciones.Anadir(aplicacion);
                Repositorio.SaveChanges();
            }
        }

        public void ModificarAplicacion(Aplicacion aplicacion)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                Repositorio.Aplicaciones.Modificar(aplicacion);
                Repositorio.SaveChanges();
            }
        }

        public Calculo ObtenerCalculoCrearSiNoExiste(short idCalculo, short idNegocio)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                Calculo calculo = null;
                if (idCalculo > 0)
                {
                    calculo = Repositorio.Calculos.Obtener(idCalculo);
                }
                else
                {
                    calculo = new Calculo()
                    {
                        IdNegocio = idNegocio,
                        FechaCreacion = DateTime.Now,
                        UsuarioCreacion = System.Environment.UserName
                    };
                    Repositorio.Calculos.Anadir(calculo);
                    Repositorio.SaveChanges();
                }

                return calculo;
            }
        }

        public Calculo ObtenerCalculo(short idNegocio)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                Calculo calculo = new Calculo();
                calculo = Repositorio.Calculos.ObtenerCalculoNegocio(idNegocio);
                return calculo;
            }
        }
        public void ModificarEscaladoCoste(Calculo_EscaladoCoste escaladoCoste)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                Repositorio.Calculo_EscaladoCostes.Modificar(escaladoCoste);
                Repositorio.SaveChanges();
            }
        }

        public List<Calculo_Criterio> ObtenerCriterios(short idCalculo)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Calculos.SeleccionarCriterios(idCalculo);
            }
        }

        public bool UsaCriterio(short idNegocio, short idCriterio)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Calculos.UsaCriterio(idNegocio, idCriterio);
            }
        }

        public List<Calculo_EscaladoCoste> ObtenerEscaladosCoste(short idCalculo)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Calculo_EscaladoCostes.Seleccionar(e => e.IdCalculo == idCalculo).OrderBy(e => e.ValorDesde).ToList();
            }
        }

        public List<string> ObtenerProcesosDisponibles(short idNegocio)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var procesosAsignados = Repositorio.Aplicaciones.SeleccionarIncluye(a => a.IdNegocio == idNegocio, a => a.Procesos).SelectMany(a => a.Procesos).Select(p => p.IdProceso).Distinct().ToList();
                return Repositorio.Procesos.SeleccionarTodos().OrderBy(p => p.IdProceso).Select(p => p.IdProceso).Except(procesosAsignados).ToList();
            }
        }

        public List<Aplicacion> ObtenerAplicaciones(short idNegocio)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Aplicaciones.SeleccionarIncluye(a => a.IdNegocio == idNegocio, p => p.Procesos, c => c.AplicacionCompatibles.Select(a => a.AplicacionCompatible)).OrderBy(a => a.Descripcion).ToList();
            }
        }
        public List<Polimero> ObtenerPolimeros()
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Polimeros.SeleccionarTodos().OrderBy(a => a.Descripcion).ToList();
            }
        }
        public List<Area> ObtenerAreas()
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Areas.SeleccionarTodos().OrderBy(a => a.Nombre).ToList();
            }
        }

        public void InsertarCriterio(short idCalculo, short idCriterio)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                Repositorio.Calculos.AnadirCriterio(idCalculo, idCriterio);
                Repositorio.SaveChanges();
            }
        }


        public void InsertarEscaladoCoste(Calculo_EscaladoCoste escaladoCoste)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                escaladoCoste.FechaCreacion = escaladoCoste.FechaModificacion = DateTime.Now;
                escaladoCoste.UsuarioCreacion = escaladoCoste.UsuarioModificacion = System.Environment.UserName;

                Repositorio.Calculos.AnadirEscaladoCoste(escaladoCoste);
                Repositorio.SaveChanges();
            }
        }

        public List<EscaladoKg> ObtenerEscaladosKg()
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.EscaladosKg.Seleccionar(e => !e.EsArticuloPulverizado).ToList();
            }
        }

        public List<Calculo_Valores> ObtenerValoresCriterioBase(short idCalculo)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.CalculoValores.Seleccionar(c => c.IdCalculo == idCalculo).ToList();
            }
        }

        public List<CriterioBaseEscaladoPrecio> ObtenerValoresCriterioBaseAjusteKg(short idCalculo)
        {
            var list = new List<CriterioBaseEscaladoPrecio>();

            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var calculo = Repositorio.Calculos.Obtener(idCalculo);

                list = Repositorio.CalculoValores.Seleccionar(c => c.IdCalculo == idCalculo).Select(v => new CriterioBaseEscaladoPrecio() { IdTipoPrecio = v.IdTipoPrecio, IdEscaladoKg = v.IdEscaladoKg }).Distinct().ToList();

                if (UsaCriterio(calculo.IdNegocio, (short)Criterios.AjusteEscaladoKg))
                {
                    list.AddRange(Repositorio.Calculos.ObtenerEscaladosPreciosCriterioAjusteKg(idCalculo));
                }

                return list;
            }
        }

        public DatosCriterioBase ObtenerValoresCriterioBase(short idCalculo, short idNegocio, int? idEscaladoCoste, short? idAplicacion, short? idEscaladoKg, short idTipoPrecio)
        {
            DatosCriterioBase dcb = new DatosCriterioBase();

            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                //Valores pivotados
                dcb.Valores = Repositorio.Calculos.SeleccionarValoresCriterioBase(idCalculo, idNegocio, idEscaladoCoste, idAplicacion, idEscaladoKg, idTipoPrecio);

                //Obtener datos columnas
                if (idEscaladoCoste.HasValue || !idEscaladoKg.HasValue)
                {
                    //Escalado Kg
                    dcb.Columnas = Repositorio.EscaladosKg.Seleccionar(e => !e.EsArticuloPulverizado && (!idEscaladoKg.HasValue || e.IdEscaladoKg == idEscaladoKg.Value)).OrderBy(e => e.Valor).ToDictionary(d => d.IdEscaladoKg, d => d.Descripcion);
                }
                else
                {
                    //Aplicaciones
                    dcb.Columnas = Repositorio.Aplicaciones.Seleccionar(a => a.IdNegocio == idNegocio && (!idAplicacion.HasValue || a.IdAplicacion == idAplicacion.Value)).OrderBy(a => a.Descripcion).ToDictionary(d => d.IdAplicacion, d => d.Descripcion);
                }

                return dcb;
            }
        }

        public void EliminarCriterio(short idCalculo, short idCriterio)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                Repositorio.Calculos.EliminarCriterio(idCalculo, idCriterio);
                Repositorio.SaveChanges();
            }
        }

        public void ActualizarValorCriterioBase(short idCalculo, int idEscaladoCoste, short idAplicacion, short idEscaladoKg, short idTipoPrecio, short idTipoValor, decimal? valor)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var list = Repositorio.CalculoValores.Seleccionar(v => v.IdCalculo == idCalculo && v.IdCalculo_EscaladoCoste == idEscaladoCoste && v.IdAplicacion == idAplicacion && v.IdEscaladoKg == idEscaladoKg && v.IdTipoPrecio == idTipoPrecio);
                Repositorio.CalculoValores.EliminarRango(list);
                if (valor.HasValue)
                {
                    Repositorio.CalculoValores.Anadir(new Calculo_Valores()
                    {
                        FechaCreacion = DateTime.Now,
                        IdAplicacion = idAplicacion,
                        IdCalculo = idCalculo,
                        IdCalculo_EscaladoCoste = idEscaladoCoste,
                        IdEscaladoKg = idEscaladoKg,
                        IdTipoPrecio = idTipoPrecio,
                        IdTipoValor = idTipoValor,
                        UsuarioCreacion = System.Environment.UserName,
                        Valor = valor.Value
                    });
                }
                Repositorio.SaveChanges();
            }
        }

        public short? ObtenerKgBase(short idCalculo)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {

                Calculo calculo = Repositorio.Calculos.Obtener(idCalculo);

                return calculo.IdEscaladoKgBase;
            }
        }

        public void ActualizarKgBase(Calculo calc, short idEscaladoKgBase)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                calc.IdEscaladoKgBase = idEscaladoKgBase;
                calc.UsuarioModificacion = System.Environment.UserName;
                calc.FechaModificacion = DateTime.Now;
                Repositorio.Calculos.Modificar(calc);
                Repositorio.SaveChanges();
            }
        }
        public DatosCriterioBase ObtenerValoresCriterioAjusteKg(short idCalculo, short idNegocio, short idTipoPrecio)
        {
            DatosCriterioBase dcb = new DatosCriterioBase();

            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                //Valores pivotados
                dcb.Valores = Repositorio.Calculos.SeleccionarValoresCriterioAjusteKg(idCalculo, idNegocio, idTipoPrecio);

                //Aplicaciones
                dcb.Columnas = Repositorio.Aplicaciones.Seleccionar(a => a.IdNegocio == idNegocio).OrderBy(a => a.Descripcion).ToDictionary(d => d.IdAplicacion, d => d.Descripcion);


                return dcb;
            }
        }
        public DatosCriterioBase ObtenerValoresCriterioZona(short idCalculo, short? idEscaladoKg, short idTipoPrecio)
        {
            DatosCriterioBase dcb = new DatosCriterioBase();

            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                //Valores pivotados
                dcb.Valores = Repositorio.Calculos.SeleccionarValoresCriterioZona(idCalculo, idTipoPrecio);

                //Obtener datos columnas

                //Zonas
                dcb.Columnas = Repositorio.EscaladosKg.Seleccionar(a => !a.EsArticuloPulverizado && (!idEscaladoKg.HasValue || a.IdEscaladoKg == idEscaladoKg.Value)).OrderBy(a => a.IdEscaladoKg).ToDictionary(d => d.IdEscaladoKg, d => d.Descripcion);


                return dcb;
            }
        }

        public DatosCriterioBase ObtenerValoresCriterioPolimero(short idCalculo, short? idEscaladoKg, short idTipoPrecio)
        {
            DatosCriterioBase dcb = new DatosCriterioBase();

            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                //Valores pivotados
                dcb.Valores = Repositorio.Calculos.SeleccionarValoresCriterioPolimero(idCalculo, idTipoPrecio);

                //Obtener datos columnas: EscaladoKg
                dcb.Columnas = Repositorio.EscaladosKg.Seleccionar(a => !a.EsArticuloPulverizado && (!idEscaladoKg.HasValue || a.IdEscaladoKg == idEscaladoKg.Value)).OrderBy(a => a.IdEscaladoKg).ToDictionary(d => d.IdEscaladoKg, d => d.Descripcion);


                return dcb;
            }
        }
        public decimal? ObtenerValoresCriterioComisionista(short idCalculo)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {

                Calculo calculo = Repositorio.Calculos.Obtener(idCalculo);

                return calculo.Comision;
            }
        }

        public DataTable ObtenerHistoricoPreciosCliente(string idArticulo, string idCliente, string moneda)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {

                var precios = Repositorio.Calculos.ObtenerHistoricoPreciosCliente(idArticulo, idCliente, moneda);

                return precios;
            }
        }

        public DataTable ObtenerHistoricoPreciosOtrosCliente(string idArticulo, string idCliente, string moneda)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {

                var precios = Repositorio.Calculos.ObtenerHistoricoPreciosOtrosCliente(idArticulo, idCliente, moneda);

                return precios;
            }
        }

        public decimal? ObtenerValoresCriterioDistribuidor(short idCalculo)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {

                Calculo calculo = Repositorio.Calculos.Obtener(idCalculo);

                return calculo.DescuentoDistribuidor;
            }
        }
        public void ActualizarValorCriterioAjusteKg(short idCalculo, short idAplicacion, short idEscaladoKg, short idTipoPrecio, short idTipoValor, decimal? valor)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var list = Repositorio.CalculoEscaladoKg.Seleccionar(v => v.IdCalculo == idCalculo && v.IdEscaladoKg == idEscaladoKg && v.IdAplicacion == idAplicacion && v.IdTipoPrecio == idTipoPrecio);
                Repositorio.CalculoEscaladoKg.EliminarRango(list);
                if (valor.HasValue)
                {
                    Repositorio.CalculoEscaladoKg.Anadir(new Calculo_EscaladoKg()
                    {
                        FechaCreacion = DateTime.Now,

                        IdCalculo = idCalculo,

                        IdEscaladoKg = idEscaladoKg,
                        IdAplicacion = idAplicacion,
                        IdTipoPrecio = idTipoPrecio,
                        IdTipoValor = idTipoValor,
                        UsuarioCreacion = System.Environment.UserName,
                        Valor = valor.Value
                    });
                }
                Repositorio.SaveChanges();
            }
        }
        public void ActualizarValorCriterioPolimero(short idCalculo, short idPolimero, short idEscaladoKg, short idTipoPrecio, short idTipoValor, decimal? valor)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var list = Repositorio.CalculoPolimero.Seleccionar(v => v.IdCalculo == idCalculo && v.IdPolimero == idPolimero && v.IdEscaladoKg == idEscaladoKg && v.IdTipoPrecio == idTipoPrecio);
                Repositorio.CalculoPolimero.EliminarRango(list);
                if (valor.HasValue)
                {
                    Repositorio.CalculoPolimero.Anadir(new Calculo_Polimero()
                    {
                        FechaCreacion = DateTime.Now,

                        IdCalculo = idCalculo,

                        IdEscaladoKg = idEscaladoKg,
                        IdPolimero = idPolimero,
                        IdTipoPrecio = idTipoPrecio,
                        IdTipoValor = idTipoValor,
                        UsuarioCreacion = System.Environment.UserName,
                        Valor = valor.Value
                    });
                }
                Repositorio.SaveChanges();
            }
        }
        public void ActualizarValorCriterioZona(short idCalculo, string idArea, short idEscaladoKg, short idTipoPrecio, short idTipoValor, decimal? valor)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var list = Repositorio.CalculoArea.Seleccionar(v => v.IdCalculo == idCalculo && v.IdArea == idArea && v.IdEscaladoKg == idEscaladoKg && v.IdTipoPrecio == idTipoPrecio);
                Repositorio.CalculoArea.EliminarRango(list);
                if (valor.HasValue)
                {
                    Repositorio.CalculoArea.Anadir(new Calculo_Area()
                    {
                        FechaCreacion = DateTime.Now,

                        IdCalculo = idCalculo,

                        IdEscaladoKg = idEscaladoKg,
                        IdArea = idArea,
                        IdTipoPrecio = idTipoPrecio,
                        IdTipoValor = idTipoValor,
                        UsuarioCreacion = System.Environment.UserName,
                        Valor = valor.Value
                    });
                }
                Repositorio.SaveChanges();
            }
        }

        public void ModificarCalculo(Calculo calculo)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                calculo.UsuarioModificacion = System.Environment.UserName;
                calculo.FechaModificacion = DateTime.Now;
                Repositorio.Calculos.Modificar(calculo);
                Repositorio.SaveChanges();
            }
        }
        public void EliminarEscaladoCoste(int idEscaladoCoste)
        {

            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var escaladoCoste = Repositorio.Calculo_EscaladoCostes.Obtener(idEscaladoCoste);
                if (escaladoCoste != null)
                {
                    //Eliminar los valores introducidos
                    var valores = Repositorio.CalculoValores.Seleccionar(c => c.IdCalculo_EscaladoCoste == idEscaladoCoste).ToList();
                    Repositorio.CalculoValores.EliminarRango(valores);
                    //Eliminar escalado
                    Repositorio.Calculo_EscaladoCostes.Eliminar(escaladoCoste);
                    Repositorio.SaveChanges();
                }
            }
        }

        public void EliminarAplicacion(short idAplicacion)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                //Eliminar los valores introducidos
                var valores = Repositorio.CalculoValores.Seleccionar(c => c.IdAplicacion == idAplicacion).ToList();
                Repositorio.CalculoValores.EliminarRango(valores);

                //TODO: Eliminar del resto de criterios

                //Eliminar aplicacion
                Repositorio.Aplicaciones.Eliminar(idAplicacion);
                Repositorio.SaveChanges();
            }
        }

        public Aplicacion ObtenerAplicacion(short idAplicacion)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Aplicaciones.Obtener(idAplicacion);
            }
        }

        public DatosPrecioCalculado CalcularPrecios(ParametrosCalculoPrecio parametros)
        {
            string warningsCosteMaterial = null;

            //Calcular costes 
            var calcCoste = new CalculadoraCoste();
            var api = new ArticuloSharedAPI();
            var costeMaterial = calcCoste.CalcularCosteMaterial(api, parametros.IdArticulo, parametros.IdPlanta, out warningsCosteMaterial);

            return CalcularPrecios(parametros, costeMaterial, warningsCosteMaterial);

        }

        public DatosPrecioCalculado CalcularPrecios(ParametrosCalculoPrecio parametros, decimal costeMaterial, string warningsCosteMaterial)
        {
            var precioRecomendado = new Dictionary<short, decimal>();

            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                //Comprobar si es producto de catalogo o ya se tiene precio vigente
                //Sera el precio recomendado
                precioRecomendado = Repositorio.Calculos.ObtenerTarifaCatalogo(parametros.IdArticulo, parametros.Moneda);
            }

            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                //Obtener datos del calculo
                ConfiguracionCalculo configCalculo = null;
                configCalculo = Repositorio.Calculos.ObtenerConfiguracionCalculo(parametros.IdNegocio, parametros.IdAplicacion, parametros.IdArea, parametros.IdPolimero);

                if (configCalculo == null)
                {
                    throw new ApplicationException("No hay configuración de cálculo");
                }

                //Ejecutar calculadora precios
                var calcPrecios = new CalculadoraPrecio();
                var precios = calcPrecios.CalcularPrecios(parametros, costeMaterial, warningsCosteMaterial, precioRecomendado, configCalculo);

                return precios;
            }
        }
    }
}
    