﻿using IQAP.Ofertas.Negocio.Datos;
using IQAP.Ofertas.Negocio.Informes;
using IQAP.Ofertas.Negocio.Modelo;
using IQAP.Ofertas.Recursos.Textos;
using IQAP.Shared.Common;
using IQAP.Shared.Negocio.API;
using IQAP.Shared.Negocio.Modelo;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Net.Mail;
using IQAP.Shared.Negocio.Dominio;
using System.Data;
using IQAP.Shared.Common.Enums;

namespace IQAP.Ofertas.Negocio.API
{
    public class OfertaAPI
    {
        private const short MesesValidezDefecto = 6;
        public Oferta CrearOferta(Negocio.Modelo.Negocio negocio, string descripcion, Cliente cliente, bool esOfertaGrupoCliente, string idGrupoCliente)
        {
            Oferta oferta = RellenarNuevaOferta(negocio, descripcion, cliente, esOfertaGrupoCliente, idGrupoCliente);

            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                Repositorio.Ofertas.Anadir(oferta);
                Repositorio.SaveChanges();
            }

            return oferta;
        }

        private Oferta RellenarNuevaOferta(Modelo.Negocio negocio, string descripcion, Cliente cliente, bool esOfertaGrupoCliente, string idGrupoCliente)
        {
            Oferta oferta = new Oferta();
            oferta.IdNegocio = negocio.IdNegocio;
            oferta.Negocio = negocio;
            oferta.ProponerDatosOferta = negocio.ProponerDatosOferta;
            oferta.Descripcion = descripcion;
            oferta.MesesValidez = ConfigurationHelper.GetShortConfiguration("OfertaMesesValidez") ?? MesesValidezDefecto;
            oferta.NumeroEnvios = 1;
            oferta.NumeroVersion = 1;
            oferta.EsOfertaGrupoCliente = esOfertaGrupoCliente;
            oferta.IdGrupoCliente = idGrupoCliente;
            oferta.IdCliente = cliente.IdCliente;
            //Forma pago y Condiciones Entrega por defecto
            if (String.IsNullOrEmpty(oferta.IdFormaPago))
            {
                oferta.IdFormaPago = cliente.CondicionesPago;
            }
            if (String.IsNullOrEmpty(oferta.IdCondicionEntrega))
            {
                oferta.IdCondicionEntrega = cliente.CondicionesEnvio;
            }
            oferta.IdRepresentante = cliente.IdRepresentante;

            //Idioma según país del cliente
            oferta.Idioma = (cliente.Idioma ?? Idiomas.Castellano).ToCultureString();

            oferta.IdMoneda = "EUR";
            oferta.FechaCreacion = DateTime.Now;
            oferta.UsuarioCreacion = System.Environment.UserName;

            var api = new UsuarioSharedAPI();
            var usuario = api.ObtenerUsuario(oferta.UsuarioCreacion);
            if (usuario != null)
            {
                oferta.Autor = usuario.Nombre;
            }

            return oferta;
        }

        public IEnumerable<Oferta> SeleccionarOfertas()
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Ofertas.Seleccionar(o=> true);
            }
        }

        public IEnumerable<FormaPago> SeleccionarFormasPago()
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.FormasPago.Seleccionar(f => f.Idioma == Idiomas.Castellano);
            }
        }

        public IEnumerable<Oferta> SeleccionarVersionesOfertas(int idOferta)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Ofertas.Seleccionar(o=> o.IdOferta == idOferta || o.IdOfertaOrigen == idOferta).OrderByDescending(o=>o.NumeroVersion);
            }
        }

        public IEnumerable<CondicionEntrega> SeleccionarCondicionesEntrega()
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.CondicionesEntrega.Seleccionar(c => c.Idioma == Idiomas.Castellano);
            }
        }

        public Oferta ObtenerOferta(int idOferta)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Ofertas.SeleccionarIncluye(o=> o.IdOferta == idOferta, o=> o.Negocio).FirstOrDefault();
            }
        }

        public List<DatosOferta> SeleccionarOfertas(int negocio, DateTime fechaDesde, DateTime fechaHasta, string descripcion, string representante, string cliente, string asistente, string articulo, bool ofertasImpresas, bool ofertasHistoricas)
        {
            using (var repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return repositorio.Ofertas.FiltrarOfertas(negocio, fechaDesde, fechaHasta, descripcion, representante, cliente, asistente, articulo, ofertasImpresas, ofertasHistoricas).ToList();
            }
        }

        public List<OfertaWebService> SeleccionarOfertasWeb(string cliente, string articulo)
        {
            using (var repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var ofertas= repositorio.Ofertas.FiltrarOfertas(0, DateTime.Now.AddYears(-10), DateTime.Now.AddYears(1), String.Empty, String.Empty, cliente, String.Empty, articulo, false, false).ToList();
                if(ofertas != null)
                {
                    var lista = new List<OfertaWebService>();
                    foreach(DatosOferta o in ofertas)
                    {
                        OfertaWebService ows = new OfertaWebService();
                        ows.IdOferta = o.IdOferta;
                        ows.Cliente = o.DescripcionCliente.Trim();
                        ows.Fecha = o.Fecha.Date;
                        ows.Articulos = o.Articulos.Trim();

                        lista.Add(ows);
                    }

                    return lista;
                }
                else
                {
                    return null;
                }
            }
        }

        public void Eliminar(int idOferta)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                Repositorio.Ofertas.EliminarOferta(idOferta);
                Repositorio.SaveChanges();
            }
        }

        public void Guardar(Oferta oferta)
        {
            if (oferta == null)
            {
                throw new ArgumentNullException();
            }
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                if (oferta.FechaImpresion.HasValue)
                {
                    Repositorio.Ofertas.GuardarHistorico(oferta.IdOferta);
                    oferta.FechaImpresion = null;
                    oferta.NumeroVersion++;
                }

                oferta.FechaModificacion = DateTime.Now;

                if(oferta.AutorModificacion == null || System.Environment.UserName!= oferta.UsuarioModificacion)
                { 
                    oferta.UsuarioModificacion = System.Environment.UserName;
                    var api = new UsuarioSharedAPI();
                    var usuario = api.ObtenerUsuario(oferta.UsuarioModificacion);
                    if (usuario != null)
                    {
                        oferta.AutorModificacion = usuario.Nombre;
                    }
                    else
                    {
                        oferta.AutorModificacion = oferta.UsuarioModificacion;
                    }
                }

                Repositorio.Ofertas.Modificar(oferta);                    
                Repositorio.SaveChanges();
            }
                       
        }
        public int GuardarLineaOferta(OfertaLinea linea)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                linea.UsuarioModificacion = System.Environment.UserName;
                linea.FechaModificacion = DateTime.Now;

                if (linea.IdOfertaLinea == 0)
                {
                    linea.UsuarioCreacion = System.Environment.UserName;
                    linea.FechaCreacion = DateTime.Now;
                    Repositorio.Ofertas.AnadirLinea(linea);
                }
                else
                {
                    Repositorio.Ofertas.ModificarLinea(linea);
                }

                Repositorio.SaveChanges();
            }

            return linea.IdOfertaLinea;
        }
        public IEnumerable<OfertaLinea> SeleccionarLineasOferta(int idOferta)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Ofertas.SeleccionarLineas(ol => ol.IdOferta==idOferta);
            }
        }

        public IEnumerable<OfertaLineaWebService> SeleccionarLineasOfertaWeb(int idOferta)
        {
            var lineas = SeleccionarLineasOferta(idOferta);
            if (lineas != null)
            {
                var lista = new List<OfertaLineaWebService>();
                foreach (OfertaLinea o in lineas)
                {
                    OfertaLineaWebService olws = new OfertaLineaWebService();
                    olws.IdLineaOferta = o.IdOfertaLinea;
                    olws.Articulo = o.IdArticulo;
                    olws.Descripcion = o.Descripcion;

                    lista.Add(olws);
                }

                return lista;
            }
            else
            {
                return null;
            }

        }
        public OfertaLinea ObtenerLineaOferta(int idOfertaLinea)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Ofertas.ObtenerLinea(idOfertaLinea);
            }
        }

        public void EliminarLinea(int idOfertaLinea)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                Repositorio.Ofertas.EliminarLinea(idOfertaLinea);
                Repositorio.SaveChanges();
            }
        }

        public bool ValidarFechasVigencia(DateTime fechaInicio, DateTime fechaFin, string idCliente, string moneda, bool esSPOT)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Ofertas.ValidarFechasVigencia(fechaInicio, fechaFin, idCliente, moneda, esSPOT);
            }
        }
        
        public void ImprimirOferta(Oferta oferta, Cliente cliente, Cliente clienteGrupo, List<string> articulosImprimir, string filename, bool imprimirFichasTecnicas = false)
        {
            string idioma = Idiomas.Castellano;
            Oferta ofertaInforme = null;
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                ofertaInforme = Repositorio.Ofertas.SeleccionarIncluye(o => o.IdOferta == oferta.IdOferta, o => o.Negocio, o => o.OfertaLineas.Select(l=>l.OfertaLineaPrecios), o=> o.ClientesExcluidos).FirstOrDefault();
                if (ofertaInforme == null)
                {
                    throw new ArgumentException("Oferta no existe.");
                }

                idioma = ofertaInforme.Idioma;
                var formaPago = Repositorio.FormasPago.Seleccionar(f=> f.IdFormaPago == oferta.IdFormaPago).OrderByDescending(o=> o.Idioma == idioma.ToM3LanguageString()).FirstOrDefault();
                var condicionesEntrega = Repositorio.CondicionesEntrega.Seleccionar(c=> c.IdCondicionEntrega == oferta.IdCondicionEntrega).OrderByDescending(o => o.Idioma == idioma).FirstOrDefault();

                //Imprimir informe en fichero PDF
                var im = new InformeManager();
                im.ImprimirOferta(ofertaInforme, articulosImprimir, cliente, clienteGrupo, formaPago, condicionesEntrega, filename);
                

                //Actualizar fecha de impresión, fechas de vigencia
                ofertaInforme.ActualizarFechas(DateTime.Now);
                Repositorio.Ofertas.Modificar(ofertaInforme);
                Repositorio.SaveChanges();

                //Refrescar oferta model
                oferta.FechaImpresion = ofertaInforme.FechaImpresion;
                oferta.FechaEntrega = ofertaInforme.FechaEntrega;
                oferta.FechaFinVigencia = ofertaInforme.FechaFinVigencia;
            }

            //Obtener valores configuración
            bool integrarOfertaM3 = ConfigurationHelper.GetBooleanConfiguration("InterfaseM3");
            bool enviarCorreo = ConfigurationHelper.GetBooleanConfiguration("EnviarMails");
            bool enviarCorreoAsistente = ConfigurationHelper.GetBooleanConfiguration("EnviarMailsAsistente");
            string mailAsistenteFake = ConfigurationHelper.GetStringConfiguration("MailAsistenteDummy");


            //Integracion a M3
            if (integrarOfertaM3)
            {
                Task.Run(() =>
                {
                    try
                    {
                        var ioM3 = new IntegradorOfertasM3();
                        ioM3.GuardarOfertaM3(oferta);
                    }
                    catch (Exception ex)
                    {
                        Logger.Error(String.Format(Mensajes.CorreoErrorOfertaTitulo, oferta.IdOferta), ex);
                        //Destinatarios
                        string destinatarios = OfertasEmailHelper.ObtenerDestinatariosEnvio(enviarCorreoAsistente, cliente.MailAsistente, mailAsistenteFake, cliente.IdRepresentante);
                        EnviarNotificacionError(destinatarios, oferta.IdOferta);
                    }
                }).ContinueWith(t =>
                {
                    Logger.Debug(String.Format("Thread integración finalizado oferta: {0}; Estado: {1} => {2}", oferta.IdOferta, t.Status, t.Exception?.Message));
                }, TaskContinuationOptions.ExecuteSynchronously);
            }

            //Enviar correo asistente
            if(enviarCorreo)
            {
                Task.Run(() =>
                {
                    //Documentos adjuntos
                    List<string> documentosAdjuntos = new List<string>() { filename };
                    if (imprimirFichasTecnicas)
                    {
                        Logger.Info(String.Format("Ha solicitado envío de FT. Oferta: {0}", ofertaInforme.IdOferta));
                        var list = ImprimirFichasTecnicas(ofertaInforme, articulosImprimir, idioma);
                        documentosAdjuntos.AddRange(list);
                    }
                    
                    //Destinatarios
                    string destinatarios = OfertasEmailHelper.ObtenerDestinatariosEnvio(enviarCorreoAsistente, cliente.MailAsistente,  mailAsistenteFake, cliente.IdRepresentante);
                    
                    //Envio
                    EnviarNotificacion(destinatarios, documentosAdjuntos, ofertaInforme);
                });
            }
        }

        public void ImprimirOfertaWeb(Oferta oferta, List<string> articulosImprimir, string destinatarios)
        {
            string filename = Path.Combine(Path.GetTempPath(), String.Format("{0}_{1:yyyyMMddhhmmss}.pdf", oferta.Descripcion.ToValidFileName(), DateTime.Now));

            string idioma = Idiomas.Castellano;
            Oferta ofertaInforme = null;

            //Busco el cliente
            var apiCliente = new ClienteSharedAPI();
            Cliente cliente = apiCliente.ObtenerCliente(oferta.IdCliente, false, true, null);
            Cliente clienteGrupo = new Cliente();
            if (oferta.EsOfertaGrupoCliente)
            {
                clienteGrupo = apiCliente.ObtenerCliente(oferta.IdGrupoCliente, true, false, null);
            }
            
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                ofertaInforme = Repositorio.Ofertas.SeleccionarIncluye(o => o.IdOferta == oferta.IdOferta, o => o.Negocio, o => o.OfertaLineas.Select(l => l.OfertaLineaPrecios), o => o.ClientesExcluidos).FirstOrDefault();
                if (ofertaInforme == null)
                {
                    throw new ArgumentException("Oferta no existe.");
                }

                idioma = ofertaInforme.Idioma;
                var formaPago = Repositorio.FormasPago.Seleccionar(f => f.IdFormaPago == oferta.IdFormaPago).OrderByDescending(o => o.Idioma == idioma.ToM3LanguageString()).FirstOrDefault();
                var condicionesEntrega = Repositorio.CondicionesEntrega.Seleccionar(c => c.IdCondicionEntrega == oferta.IdCondicionEntrega).OrderByDescending(o => o.Idioma == idioma).FirstOrDefault();

                //Imprimir informe en fichero PDF
                var im = new InformeManager();
                im.ImprimirOferta(ofertaInforme, articulosImprimir, cliente, clienteGrupo, formaPago, condicionesEntrega, filename);

                //Enviar correo asistente

                Task.Run(() =>
                {
                    //Documentos adjuntos
                    List<string> documentosAdjuntos = new List<string>() { filename };

                    //Envio
                    EnviarNotificacion(destinatarios, documentosAdjuntos, ofertaInforme);
                });                

            }            
        }

        public List<OfertaClienteGrupo> SeleccionarClienteGrupo(int idOferta, string grupo)
        {
            var api = new ClienteSharedAPI();

            var list = api.SeleccionarClientesGrupo(grupo);
            //Obtener los clientes excluidos
            var clientesExcluidos = new List<string>();
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                clientesExcluidos = Repositorio.OfertaClienteExcluidos.Seleccionar(c => c.IdOferta == idOferta).Select(c=> c.IdCliente).ToList();
            }

            return list.Select(i => new OfertaClienteGrupo()
                        {
                            IdCliente = i.IdCliente,
                            Nombre = i.Nombre,
                            EstaExcluidoOferta = clientesExcluidos.Contains(i.IdCliente)
                        }).ToList();
        }

        public void ExcluirClienteOferta(int idOferta, string idCliente)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var clienteExcluido = new OfertaClienteExcluido()
                {
                    IdOferta = idOferta,
                    IdCliente = idCliente,
                    FechaCreacion = DateTime.Now,
                    UsuarioCreacion = Environment.UserName
                };

                Repositorio.OfertaClienteExcluidos.Anadir(clienteExcluido);
                Repositorio.SaveChanges();
            }
        }

        public void IncluirClienteOferta(int idOferta, string idCliente)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var clienteExcluido = Repositorio.OfertaClienteExcluidos.Seleccionar(c => c.IdOferta == idOferta && c.IdCliente == idCliente).FirstOrDefault();
                if(clienteExcluido!=null)
                {
                    Repositorio.OfertaClienteExcluidos.Eliminar(clienteExcluido);
                    Repositorio.SaveChanges();
                }
            }
        }


       
        private void EnviarNotificacion(string destinatarios, List<string> documentosAdjuntos, Oferta ofertaInforme)
        {
            var notificador = new NotificadorOferta();
            var mensajeBuilder = new OfertaImprimirMensajeConstructor(ofertaInforme, documentosAdjuntos);
            using (var mensaje = EmailExtensions.CrearMensajeEmail(destinatarios, null, null, mensajeBuilder.ObtenerTitulo(), mensajeBuilder.ObtenerMensaje(), documentosAdjuntos))
            {
                var canalEmail = new CanalNotificacionEmailServer(mensaje, Aplicaciones.Ofertas);
                notificador.Notificar(canalEmail);
            }
        }

        private void EnviarNotificacionError(string destinatarios, int idOferta)
        {
            var notificador = new NotificadorOferta();
            using (var mensaje = EmailExtensions.CrearMensajeEmail(destinatarios, null, null, String.Format(Mensajes.CorreoErrorOfertaTitulo,idOferta), String.Format(Mensajes.CorreoErrorOfertaTexto,idOferta), null))
            {
                var canalEmail = new CanalNotificacionEmailServer(mensaje, Aplicaciones.Ofertas);
                notificador.Notificar(canalEmail);
            }
        }

        private List<string> ImprimirFichasTecnicas(Oferta oferta, List<string> articulosImprimir, string idioma)
        {
            var listaRet = new List<string>();

            var api = new DocumentacionTecnicaSharedAPI();

            //Recuperar las lineas si no estan
            if (oferta.OfertaLineas == null)
            {
                using (var Repositorio = new RepositorioManager(new OfertasEntities()))
                {
                    oferta = Repositorio.Ofertas.SeleccionarIncluye(o => o.IdOferta == oferta.IdOferta, o => o.OfertaLineas).FirstOrDefault();
                }
            }
            var listaLineas = oferta.OfertaLineas ?? new List<OfertaLinea>();
            if (articulosImprimir != null && articulosImprimir.Count > 0)
            {
                listaLineas = listaLineas.Where(l => articulosImprimir.Contains(l.IdArticulo)).ToList();
            }

            foreach (string codigoArticulo in listaLineas.Select(l=>l.IdArticulo))
            {
                try
                {
                    var doc = api.ObtenerFichaTecnica(codigoArticulo, idioma);

                    if (doc != null && doc.Length > 0)
                    {
                        GuardarFicheroFT(idioma, listaRet, codigoArticulo, doc);
                    }
                }
                catch (InvalidOperationException ex)
                {
                    string txtError = String.Format(Mensajes.ObtenerFichaTecnicaInvalid, codigoArticulo);
                    Logger.Error(txtError, ex);

                    GuardarFicheroFTError(idioma, listaRet, codigoArticulo, txtError);

                }
                catch (Exception ex)
                {
                    string txtError = Mensajes.ErrorObtenerFichaTecnica;
                    Logger.Error(txtError, ex);

                    GuardarFicheroFTError(idioma, listaRet, codigoArticulo, txtError);
                }
            }

            return listaRet;
        }

        private void GuardarFicheroFT(string idioma, List<string> listaRet, string codigoArticulo, byte[] doc)
        {
            string path = Path.Combine(Path.GetTempPath(), string.Format("FT_{0}_{1}_{2:yyyyMMddhhmmss}.pdf", codigoArticulo, idioma, DateTime.Now));
            doc.GuardarFichero(path);
            listaRet.Add(path);
        }

        private void GuardarFicheroFTError(string idioma, List<string> listaRet, string codigoArticulo, string txtError)
        {
            string path = Path.Combine(Path.GetTempPath(), string.Format("Error_FT_{0}_{1}_{2:yyyyMMddhhmmss}.txt", codigoArticulo, idioma, DateTime.Now));
            txtError.GuardarTextoFichero(path);
            listaRet.Add(path);
        }

        public List<int> ObtenerLineasOfertaConCambiosRespectoVersionAnterior(int idOferta)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var list = Repositorio.Ofertas.SeleccionarLineasCambiosVersionAnterior(idOferta);
                return list;
            }
        }

        public void CambiarMonedaOferta(Moneda divisaOrigen, Oferta oferta)
        {
            if (oferta == null)
            {
                throw new ArgumentNullException();
            }
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                if (oferta.FechaImpresion.HasValue)
                {
                    Repositorio.Ofertas.GuardarHistorico(oferta.IdOferta);
                    oferta.FechaImpresion = null;
                    oferta.NumeroVersion++;
                }

                oferta.FechaModificacion = DateTime.Now;
                oferta.UsuarioModificacion = System.Environment.UserName;

                decimal factor = (divisaOrigen.ValorCambio ?? 0) > 0? (oferta.CambioDivisa?? 1) / divisaOrigen.ValorCambio ?? 1 : (oferta.CambioDivisa ?? 1);
                Repositorio.Ofertas.CambioDivisa(oferta.IdOferta, factor);

                Repositorio.Ofertas.Modificar(oferta);

                Repositorio.SaveChanges();
            }
        }

        public List<PrecioEscaladoTarifa> ObtenerPreciosTarifaLineas(int idOferta)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                return Repositorio.Ofertas.ObtenerPreciosTarifaLineas(idOferta);
            }
        }

        /// <summary>
        /// Devuelve las aplicaciones que sean compatibles con la escogida segun el proceso. La escogida se devuelve en primera posicion
        /// </summary>
        /// <param name="idNegocio"></param>
        /// <param name="idProceso"></param>
        /// <returns></returns>
        public List<Aplicacion> ObtenerAplicacionesCompatibles(short idNegocio, string idProceso)
        {
            List<Aplicacion> list = new List<Aplicacion>();
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var aplicacionProceso = Repositorio.Aplicaciones.SeleccionarIncluye(a => a.IdNegocio == idNegocio && a.Procesos.Select(p => p.IdProceso).Contains(idProceso), i=> i.AplicacionCompatibles).FirstOrDefault();
                if (aplicacionProceso != null)
                {
                    var listIdCompatibles = aplicacionProceso.AplicacionCompatibles.Select(c => c.IdAplicacionCompatible).ToList();
                    list.Add(aplicacionProceso);
                    var listAplicacionesCompatibles = Repositorio.Aplicaciones.Seleccionar(a => a.IdNegocio == idNegocio && listIdCompatibles.Contains(a.IdAplicacion)).OrderBy(a => a.Descripcion).ToList();
                    list.AddRange(listAplicacionesCompatibles);
                }

                return list;
            }
        }

        public Oferta CrearOfertaSeleccionaTarifas(DataTable dtTarifasNuevaOferta, Negocio.Modelo.Negocio negocio)
        {
            if(dtTarifasNuevaOferta.Rows.Count>0)
            {
                string idCliente = dtTarifasNuevaOferta.Rows[0]["Cliente"].ToString();
                var apiCliente = new ClienteSharedAPI();
                var cliente = apiCliente.ObtenerCliente(idCliente, false, true, null);
                if(cliente == null)
                {
                    throw new ApplicationException("Cliente no existe!");
                }
                
                string descripcion = String.Format("{0} {1:dd-MM-yyyy}", cliente.Nombre, DateTime.Now);
                var oferta = RellenarNuevaOferta(negocio, descripcion, cliente, false, cliente.Grupo);

                if (oferta != null)
                {
                    foreach (DataRow row in dtTarifasNuevaOferta.Rows)
                    {
                       var ofertaLinea = new OfertaLinea()
                        {
                            CosteMaterial = (decimal)row["Coste"],
                            FechaCreacion = oferta.FechaCreacion,
                            GrupoProducto = row["GrupoProducto"].ToString(),
                            GrupoTecnologico = row["GrupoTecnologico"].ToString(),
                            IdArticulo = row["IdArticulo"].ToString(),
                            Descripcion = row["ArticuloDescripcion"].ToString(),
                            IdClienteFinal = !String.IsNullOrEmpty(row["ClienteFinal"].ToString()) ? row["ClienteFinal"].ToString() : null,
                            EsArticuloPulverizado = EsRowPulverizado(dtTarifasNuevaOferta, row),
                            UsuarioCreacion = oferta.UsuarioCreacion,
                            OfertaLineaPrecios = new List<OfertaLineaPrecio>()
                        };
                        
                        foreach(short idEscaladoKg in ListaEscaladosKg.EscaladosKgPulverizados)
                        {
                            if(dtTarifasNuevaOferta.Columns.Contains(idEscaladoKg.ToString()) && row[idEscaladoKg.ToString()]!=DBNull.Value)
                            {
                                var ofertaLineaPrecio = new OfertaLineaPrecio()
                                {
                                    IdEscaladoKg = idEscaladoKg,
                                    Valor = (decimal)row[idEscaladoKg.ToString()]
                                };
                                ofertaLinea.OfertaLineaPrecios.Add(ofertaLineaPrecio);
                            }
                        }

                        oferta.OfertaLineas.Add(ofertaLinea);
                    }

                    using (var Repositorio = new RepositorioManager(new OfertasEntities()))
                    {
                        Repositorio.Ofertas.Anadir(oferta);
                        Repositorio.SaveChanges();
                    }

                    return oferta;
                }
            }

            return null;
        }

        private bool EsRowPulverizado(DataTable dt, DataRow row)
        {
            bool esPulverizado = false;
            foreach (short idEscaladoKg in ListaEscaladosKg.EscaladosKgPulverizados)
            {
                if(ListaEscaladosKg.EsEscaladoPulverizado(idEscaladoKg))
                {
                    if(!dt.Columns.Contains(idEscaladoKg.ToString()))
                    {
                        return false;
                    }
                    else
                    {
                        if(row[idEscaladoKg.ToString()] != DBNull.Value)
                        {
                            esPulverizado = true;
                        }
                    }
                }
            }
            return esPulverizado;
        }

        public void ModificarDescripcionAdicionalLineaOferta(int idOfertaLinea, string descripcion)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var linea = Repositorio.Ofertas.ObtenerLinea(idOfertaLinea);
                linea.DescripcionAdicional = descripcion;
                Repositorio.SaveChanges();
            }
        }

        public void ModificarPrecioUnicoLineaOferta(int idOfertaLinea, decimal? precio, decimal? importeEstimado, decimal? VAEstimado)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var linea = Repositorio.Ofertas.ObtenerLinea(idOfertaLinea);
                linea.PrecioUnico = precio;
                linea.ImporteEstimado = importeEstimado;
                linea.VAEstimado = VAEstimado;
                Repositorio.SaveChanges();
            }
        }

        public void ModificarPrecioLineaOferta(int idOfertaLinea, short idEscaladoKg, decimal? valor, decimal? importeEstimado, decimal? VAEstimado)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var precio = Repositorio.Ofertas.ObtenerLineaPrecio(idOfertaLinea, idEscaladoKg);
                if (valor.HasValue)
                {
                    if (precio != null)
                    {
                        precio.Valor = valor.Value;
                    }
                    else
                    {
                        Repositorio.Ofertas.AnadirLineaPrecio(new OfertaLineaPrecio() { IdOfertaLinea = idOfertaLinea, IdEscaladoKg = idEscaladoKg, Valor = valor.Value });
                    }
                }
                else
                {
                    if (precio != null)
                    {
                        Repositorio.Ofertas.EliminarLineaPrecio(precio);
                    }
                }
                //Actualizar importe Estimado y VAEstimado
                var linea = Repositorio.Ofertas.ObtenerLinea(idOfertaLinea);
                if(linea!=null)
                {
                    linea.ImporteEstimado = importeEstimado;
                    linea.VAEstimado = VAEstimado;
                }
                Repositorio.SaveChanges();
            }
        }

        public void ModificarCantidadEstimadaLineaOferta(int idOfertaLinea, decimal? valor, decimal? importeEstimado, decimal? VAEstimado)
        {
            using (var Repositorio = new RepositorioManager(new OfertasEntities()))
            {
                var linea = Repositorio.Ofertas.ObtenerLinea(idOfertaLinea);
                if (linea != null)
                {
                    linea.CantidadEstimada = valor;
                    linea.ImporteEstimado = importeEstimado;
                    linea.VAEstimado = VAEstimado;
                    Repositorio.SaveChanges();
                }
            }
        }
    }
}
