﻿using IQAP.Ofertas.Negocio.Datos;
using IQAP.Ofertas.Negocio.Informes;
using IQAP.Ofertas.Negocio.Modelo;
using IQAP.Ofertas.Recursos.Textos;
using IQAP.Shared.Common;
using IQAP.Shared.Common.Data;
using IQAP.Shared.Negocio.API;
using IQAP.Shared.Negocio.Dominio;
using IQAP.Shared.Negocio.Modelo;
using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IQAP.Ofertas.Negocio.API
{
    public class TarifaAPI
    {
        private const int PrimeraColumnaPrecioTarifaComparativaVA = 21;
        private const string TipoCosteComparativaVA = "4";
        private const decimal MermaCosteComparativaVA = 0.05m;

        public DataTable ConsultarTarifas(FiltroConsultaTarifas filtro)
        {
            DataTable dt = null;

            using (var tarifaDP = new TarifaDataProvider(new OfertasEntities()))
            {
                dt = tarifaDP.ConsultarTarifas(filtro);
            }

            //Calcular coste y descripción artículo
            dt = CalcularInfoArticuloExtraConsultaTarifas(dt, filtro.VerCoste, filtro.VerHistorico);

            //Filtrar las filas que no aplican
            if(filtro.VerSoloTarifasAplica && dt.Rows.Count>0)
            {
                dt = dt.AsEnumerable().Where(r => !r.Field<bool>("NoAplica")).CopyToDataTable();
            }

            return dt;

        }

        public void ImprimirConsultaTarifa(DataTable dtTarifas, int columnaPrimerEscaladoKg, FiltroConsultaTarifas filtro,  string filename, bool verCabecera, bool agruparClienteFinal, string idioma)
        {
            //Imprimir informe en fichero PDF
            var im = new InformeManager();
            im.ImprimirTarifas(dtTarifas, columnaPrimerEscaladoKg, filtro, filename, verCabecera, agruparClienteFinal, idioma);
        }

        private DataTable CalcularInfoArticuloExtraConsultaTarifas(DataTable dt, bool verCoste, bool verHistorico)
        {
            Dictionary<string, InfoArticuloExtraConsultaTarifas> dictArticulo = new Dictionary<string, InfoArticuloExtraConsultaTarifas>();
            var apiArticulo = new ArticuloSharedAPI();
            foreach(DataRow row in dt.Rows)
            {
                string idArticulo = row["IdArticulo"].ToString();
                string idArticuloInterno = row["ArticuloInterno"].ToString();
                string descripcionArticulo = row["ArticuloDescripcion"].ToString();
                string descripcionArticuloCliente = row["ArticuloClienteDescripcion"].ToString();
                decimal? coste = null;
                short idPlanta = 1;

                if (dictArticulo.ContainsKey(idArticulo))
                {
                    coste = dictArticulo[idArticulo].Coste;
                    descripcionArticulo =  dictArticulo[idArticulo].DescripcionComercial;
                }
                else
                {
                    //Asegurarse que el articulo interno tiene valor
                    if(String.IsNullOrEmpty(idArticuloInterno))
                    {
                        idArticuloInterno = idArticulo;
                    }

                    if (verCoste)
                    {
                        //Calculo de coste
                        idPlanta = apiArticulo.ObtenerPlanta(idArticuloInterno);
                        string warningsCosteMaterial = null;
                        coste = apiArticulo.CosteMateriales(idArticuloInterno, idPlanta, out warningsCosteMaterial);
                    }

                    //Calculo de descripcion comercial
                    descripcionArticulo = apiArticulo.NombreComercial(idArticuloInterno, row["GrupoTecnologico"].ToString(), row["SubgrupoTecnologico"].ToString(), row["GrupoProducto"].ToString(), descripcionArticulo);

                    //Guardar en diccionario
                    dictArticulo.Add(idArticulo, new InfoArticuloExtraConsultaTarifas() { Coste = coste, DescripcionComercial = descripcionArticulo });
                }

                if (coste.HasValue)
                {
                    row["Coste"] = coste;
                }

                //Articulo descripcion
                row["ArticuloDescripcion"] = String.IsNullOrWhiteSpace(descripcionArticuloCliente) ? descripcionArticulo: descripcionArticuloCliente;

                //NoAplica tarifa
                row["NoAplica"] =   (!verHistorico &&
                                    ((row.Field<string>("TipoTarifa") == "01" &&
                                        dt.AsEnumerable().Any(p => p.Field<string>("TipoTarifa") != "01" &&
                                                                                            p.Field<string>("IdArticulo") == row.Field<string>("IdArticulo") &&
                                                                                            p.Field<string>("Cliente") == row.Field<string>("Cliente") &&
                                                                                            p.Field<string>("Moneda") == row.Field<string>("Moneda"))) ||
                                    (row.Field<string>("TipoTarifa") == "OF" &&
                                        dt.AsEnumerable().Any(p => p.Field<string>("TipoTarifa") == "SP" &&
                                                                                            p.Field<string>("IdArticulo") == row.Field<string>("IdArticulo") &&
                                                                                            p.Field<string>("Cliente") == row.Field<string>("Cliente") &&
                                                                                            p.Field<string>("Moneda") == row.Field<string>("Moneda")))));
            }

            return dt;
        }

        public class InfoArticuloExtraConsultaTarifas
        {
            public decimal? Coste { get; set; }
            public string DescripcionComercial { get; set; }
        }

        public DataTable ConsultarComparativaVA(FiltroConsultaComparativaVA filtro)
        {
            DataTable dt = null;

            using (var tarifaDP = new TarifaDataProvider(new OfertasEntities()))
            {
                dt = tarifaDP.ConsultarComparativaVA(filtro);
            }

            //Calcular coste, coste original, VA y coeficientes
            dt = CalcularInfoConsultaComparativaVA(dt, filtro.Fecha, filtro.VerSeleccionado, filtro.VerCantidadVA);

            //Filtrar las filas segun Ver seleccionado
            if (dt.Rows.Count > 0)
            {
                dt = FiltrarInfoConsultaComparativaVA(filtro, dt);
            }

            return dt;
        }

        private DataTable FiltrarInfoConsultaComparativaVA(FiltroConsultaComparativaVA filtro, DataTable dt)
        {
            EnumerableRowCollection<DataRow> rows = null;
            //Filtrar los más importantes
            switch (filtro.VerSeleccionado)
            {
                case (short)VerComparativaVA.DiferenciaEuroKg:
                    rows = dt.AsEnumerable().Where(r => r.Field<decimal>("DiferenciaMaximaEurKg") > filtro.DiferenciaEurKgImportante);
                    break;
                case (short)VerComparativaVA.DiferenciaEuroKgNegativa:
                    rows = dt.AsEnumerable().Where(r => r.Field<decimal>("DiferenciaMaximaEurKg") < -filtro.DiferenciaEurKgImportante);
                    break;
                case (short)VerComparativaVA.DiferenciaVA:
                    rows = dt.AsEnumerable().Where(r => r.Field<decimal>("DiferenciaMaximaVA") > filtro.DiferenciaVAImportante);
                    break;
                case (short)VerComparativaVA.DiferenciaVANegativa:
                    rows = dt.AsEnumerable().Where(r => r.Field<decimal>("DiferenciaMaximaVA") < -filtro.DiferenciaVAImportante);
                    break;
                default:
                    break;
            }

            if (filtro.VerSeleccionado != (short)VerComparativaVA.Todos)
            {
                if (rows.Any())
                {
                    dt = rows.CopyToDataTable();
                }
                else
                {
                    dt.Rows.Clear();
                }
            }

            return dt;
        }

        private DataTable CalcularInfoConsultaComparativaVA(DataTable dt, DateTime? fechaFiltro, short verSeleccionado, bool verCantidadVA)
        {
            var apiArticulo = new ArticuloSharedAPI();
            var calculadoraCoste = new CalculadoraCoste();

            foreach (DataRow row in dt.Rows)
            {
                CalcularInfoConsultaComparativaVARow(fechaFiltro, verSeleccionado, verCantidadVA, apiArticulo, calculadoraCoste, row);
            }
            return dt;
        }

        private static void CalcularInfoConsultaComparativaVARow(DateTime? fechaFiltro, short verSeleccionado, bool verCantidadVA, ArticuloSharedAPI apiArticulo, CalculadoraCoste calculadoraCoste, DataRow row)
        {
            string idArticulo = row["IdArticulo"].ToString();
            string idArticuloInterno = row["ArticuloInterno"].ToString();
            string descripcionArticulo = row["ArticuloDescripcion"].ToString();
            string descripcionArticuloCliente = row["ArticuloClienteDescripcion"].ToString();
            //Prevalece la fecha indicada por parámetro (sino fecha inicio).
            DateTime fechaInicioTarifa = fechaFiltro ?? (DateTime)row["FechaInicio"];
            decimal? coste = null;
            decimal? costeInicial = null;
            decimal maximaDiferenciaEurKg = 0;
            decimal maximaDiferenciaVA = 0;

            //Asegurarse que el articulo interno tiene valor
            if (String.IsNullOrEmpty(idArticuloInterno))
            {
                idArticuloInterno = idArticulo;
            }

            //Calculo de descripcion comercial
            descripcionArticulo = apiArticulo.NombreComercial(idArticuloInterno, row["GrupoTecnologico"].ToString(), row["SubgrupoTecnologico"].ToString(), row["GrupoProducto"].ToString(), descripcionArticulo);
            //Articulo descripcion
            row["ArticuloDescripcion"] = String.IsNullOrWhiteSpace(descripcionArticuloCliente) ? descripcionArticulo : descripcionArticuloCliente;

            //Calculo de coste
            coste = calculadoraCoste.CalcularCosteMaterial(idArticuloInterno, TipoCosteComparativaVA, "FAB", DateTime.Now, true, MermaCosteComparativaVA);
            costeInicial = calculadoraCoste.CalcularCosteMaterial(idArticuloInterno, TipoCosteComparativaVA, "FAB", fechaInicioTarifa, true, MermaCosteComparativaVA);

            row["Coste"] = coste.ToADOParameterValue();
            row["CosteInicial"] = costeInicial.ToADOParameterValue();

            //VA y VA inicial (3 columnas - Precio, VA, VA Inicial)
            for (int col = PrimeraColumnaPrecioTarifaComparativaVA; col < row.ItemArray.Length; col += 3)
            {
                decimal? precio = !string.IsNullOrEmpty(row[col].ToString()) ? (decimal?)row[col] : null;
                decimal? valorVA = calculadoraCoste.CalcularVA(precio, coste, false);
                decimal? valorVAInicial = calculadoraCoste.CalcularVA(precio, costeInicial, false);
                decimal diferenciaVA = 0;
                
                if (valorVA.HasValue && valorVAInicial.HasValue)
                {
                    diferenciaVA = valorVAInicial.Value - valorVA.Value;
                    
                    if (Math.Abs(diferenciaVA) > Math.Abs(maximaDiferenciaVA))
                    {
                        maximaDiferenciaVA = diferenciaVA;
                    }
                }

                //Calcular importe VA si se desea ver el importe VA
                if(verCantidadVA)
                {
                    valorVA = calculadoraCoste.CalcularVA(precio, coste, verCantidadVA);
                    valorVAInicial = calculadoraCoste.CalcularVA(precio, costeInicial, verCantidadVA);
                }

                row[col + 1] = valorVA.ToADOParameterValue();
                row[col + 2] = valorVAInicial.ToADOParameterValue();
            }

            if (coste.HasValue && costeInicial.HasValue)
            {
                maximaDiferenciaEurKg = costeInicial.Value - coste.Value;
            }

            //Diferencias máximas Eur/Kg y VA(%)
            row["DiferenciaMaximaEurKg"] = maximaDiferenciaEurKg;
            row["DiferenciaMaximaVA"] = maximaDiferenciaVA;
        }

        public void IncrementarPrecios(DataTable dtPrecios, Cliente cliente, DateTime fechaInicio, DateTime fechaFin, int columnaPrimerEscalado, string filename, string idioma)
        {
            if (dtPrecios!=null && dtPrecios.Rows.Count==0)
            {
                throw new ArgumentException("Tabla de precios vacía.");
            }
            //Rellenar Incremento Precios
            var incremento = new IncrementoPrecio()
            {
                GrupoCliente = dtPrecios.Rows[0]["GrupoCliente"].ToString(),
                GrupoClienteDescripcion = dtPrecios.Rows[0]["GrupoClienteDescripcion"].ToString(),
                Cliente = dtPrecios.Rows[0]["Cliente"].ToString(),
                ClienteDescripcion = dtPrecios.Rows[0]["ClienteDescripcion"].ToString(),
                ClienteFinal = dtPrecios.Rows[0]["ClienteFinal"].ToString(),
                ClientFinalDescripcion = dtPrecios.Rows[0]["ClienteFinalDescripcion"].ToString(),
                NuevaFechaInicio= fechaInicio,
                NuevaFechaFin= fechaFin,
                Moneda = dtPrecios.Rows[0]["Moneda"].ToString(),
                UsuarioCreacion = System.Environment.UserName,
                FechaCreacion = DateTime.Now,
                Articulos = new List<IncrementoPreciosArticulo>()
            };

            var lineas = new List<IncrementoPreciosLinea>();
            foreach (DataRow row in dtPrecios.Rows)
            {
                var lineasArticulo = new List<IncrementoPreciosLinea>();
                var incrementoArticulo = new IncrementoPreciosArticulo()
                {
                    IdArticulo = row["IdArticulo"].ToString(),
                    ArticuloDescripcion = row["ArticuloDescripcion"].ToString(),
                    FechaInicio = (DateTime) row["FechaInicio"],
                    FechaFin = (DateTime) row["FechaFin"],
                    Coste = (decimal)row["Coste"]
                };

                for(int col= columnaPrimerEscalado; col < dtPrecios.Columns.Count; col+=2)
                {
                    if (row[col]!=DBNull.Value && 
                        decimal.TryParse(row[col].ToString(), out decimal precio) && 
                        short.TryParse(dtPrecios.Columns[col].ColumnName, out short idEscaladoKg))
                    {
                        lineasArticulo.Add(new IncrementoPreciosLinea()
                        {
                            IdArticulo = row["IdArticulo"].ToString(),
                            IdEscaladoKg = idEscaladoKg,
                            Precio = precio
                        });
                    }
                }

                incrementoArticulo.PrecioBase = lineasArticulo.Max(p => p.Precio);
                incremento.Articulos.Add(incrementoArticulo);
                lineas.AddRange(lineasArticulo);
            }

            incremento.Lineas = lineas;


            using (var tarifaDP = new TarifaDataProvider(new OfertasEntities()))
            {
                tarifaDP.GuardarIncrementarPrecios(incremento);
                tarifaDP.IncrementarPrecios(incremento.IdIncrementoPrecios);
            }

            //Imprimir
            ImprimirIncrementoPrecios(dtPrecios, cliente, columnaPrimerEscalado, filename, idioma);
        }

        public bool ExisteTarifaFutura(string cliente, string moneda, DateTime fechaInicio, out DateTime? fechaInicioFutura)
        {
            fechaInicioFutura = null;
            DataTable dt;
            using (var tarifaDP = new TarifaDataProvider(new OfertasEntities()))
            {
                dt = tarifaDP.ObtenerTarifasFuturas(cliente, moneda, fechaInicio);
                if(dt.Rows.Count>0)
                {
                    string fechaString = dt.AsEnumerable().Min(r => int.Parse(r["OJFVDT"].ToString())).ToString();
                    fechaInicioFutura = DateTime.ParseExact(fechaString, "yyyyMMdd", CultureInfo.InvariantCulture);
                }
                else
                {
                    fechaInicioFutura = null;
                }
            }

            return dt.Rows.Count > 0;
        }

        private void ImprimirIncrementoPrecios(DataTable dtPrecios, Cliente cliente,  int columnaPrimerEscaladoKg, string filename, string idioma)
        {
            var im = new InformeManager();
            im.ImprimirIncrementoPrecios(dtPrecios, cliente, columnaPrimerEscaladoKg, filename, idioma);

            bool enviarCorreo = ConfigurationHelper.GetBooleanConfiguration("EnviarMails");
            bool enviarCorreoAsistente = ConfigurationHelper.GetBooleanConfiguration("EnviarMailsAsistente");
            string mailAsistenteFake = ConfigurationHelper.GetStringConfiguration("MailAsistenteDummy");
            //Enviar correo asistente
            if (enviarCorreo)
            {
                Task.Run(() =>
                {
                    //Documentos adjuntos
                    List<string> documentosAdjuntos = new List<string>() { filename };
                    //Destinatarios
                    string destinatarios = OfertasEmailHelper.ObtenerDestinatariosEnvio(enviarCorreoAsistente, cliente.MailAsistente, mailAsistenteFake, cliente.IdRepresentante);
                    //Envio
                    EnviarNotificacion(destinatarios, documentosAdjuntos, cliente.Nombre);
                });
            }

        }
      
        private void EnviarNotificacion(string destinatarios, List<string> documentosAdjuntos, string cliente)
        {
            var notificador = new NotificadorOferta();
            var mensajeBuilder = new IncrementoPrecioImprimirMensajeConstructor(cliente, documentosAdjuntos);
            using (var mensaje = EmailExtensions.CrearMensajeEmail(destinatarios, null, null, mensajeBuilder.ObtenerTitulo(), mensajeBuilder.ObtenerMensaje(), documentosAdjuntos))
            {
                var canalEmail = new CanalNotificacionEmailServer(mensaje, Aplicaciones.Ofertas);
                notificador.Notificar(canalEmail);
            }
        }

        public void AnadirColumnasVA(DataTable dt, int columnaIdArticulo, int columnaIdArticuloInterno, int columnaCoste, int columnaPrimerEscaladoKg)
        {
            //Crear columnas
            int numColumnasNuevas = 0;
            int numColumnas = dt.Columns.Count;
            for (int col = columnaPrimerEscaladoKg; col < numColumnas; col++)
            {
                dt.Columns.Add(dt.Columns[col + numColumnasNuevas].ColumnName + "VA", typeof(decimal))
                    .SetOrdinal(col + (++numColumnasNuevas));
            }

            CalcularColumnaCoste(dt, columnaIdArticulo, columnaIdArticuloInterno, columnaCoste);
            CalcularColumnasVA(dt, columnaCoste, columnaPrimerEscaladoKg, false);
        }

        public void CalcularColumnaCoste(DataTable dt, int columnaIdArticulo, int columnaIdArticuloInterno, int columnaCoste)
        {
            var calculadoraCoste = new CalculadoraCoste();
            //Calcular valores
            foreach (DataRow row in dt.Rows)
            {
                decimal? coste = !String.IsNullOrEmpty(row[columnaCoste].ToString()) ? (decimal?)row[columnaCoste] : null;
                if (!coste.HasValue)
                {
                    string articuloInterno = !String.IsNullOrEmpty(row[columnaIdArticuloInterno].ToString()) ? row[columnaIdArticuloInterno].ToString() : row[columnaIdArticulo].ToString();
                    coste = calculadoraCoste.CalcularCosteMaterial(articuloInterno, TipoCosteComparativaVA, "FAB", DateTime.Now, true, MermaCosteComparativaVA);
                    row[columnaCoste] = coste;
                }
            }
        }
        public void CalcularColumnasVA(DataTable dt, int columnaCoste, int columnaPrimerEscaladoKg, bool cantidadVA)
        {
            var calculadoraCoste = new CalculadoraCoste();
            //Calcular valores
            foreach (DataRow row in dt.Rows)
            {
                decimal? coste = !String.IsNullOrEmpty(row[columnaCoste].ToString()) ? (decimal?)row[columnaCoste] : null;
                for (int col = columnaPrimerEscaladoKg; col < row.ItemArray.Length; col += 2)
                {
                    decimal? precio = !string.IsNullOrEmpty(row[col].ToString()) ? (decimal?)row[col] : null;
                    decimal? valorVA = calculadoraCoste.CalcularVA(precio, coste, cantidadVA);

                    row[col + 1] = valorVA.ToADOParameterValue();
                }
            }
        }

        public decimal? CalcularVA(decimal? importe, decimal? coste, bool cantidadVA)
        {
            var calculadoraCoste = new CalculadoraCoste();
            return calculadoraCoste.CalcularVA(importe, coste, cantidadVA);
        }

        public bool ExistenTarifasIncrementoPrecios(string cliente, DateTime fechaInicio, DateTime fechaFin)
        {
            using (var tarifaDP = new TarifaDataProvider(new OfertasEntities()))
            {
                return tarifaDP.ExistenTarifasIncrementoPrecios(cliente, fechaInicio, fechaFin);
            }
        }
    }
}
