﻿using IQAP.Ofertas.Negocio.API;
using IQAP.Ofertas.Negocio.Datos;
using IQAP.Ofertas.Negocio.Modelo;
using IQAP.Shared.Negocio;
using IQAP.Shared.Negocio.API;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IQAP.Ofertas.Negocio
{
    public class CalculadoraCoste
    {
        /// <summary>
        /// Implementa una cache estatica para los costes
        /// </summary>
        private static Dictionary<string, decimal> cacheCoste = new Dictionary<string, decimal>();
        public decimal CalcularCosteMaterial(IArticuloSharedAPI api, string idArticulo, short idPlanta, out string warnings)
        {
            var costeMaterial = api.CosteMateriales(idArticulo, idPlanta, out warnings);

            if(idPlanta == (int) Plantas.Czech && costeMaterial>0)
            {
                //La moneda es CZK, hay que cambiar la divisa a EUR
                var apiMoneda = new DivisaSharedAPI();
                costeMaterial = apiMoneda.ObtenerImporteOtraDivisa(costeMaterial, "CZK", "EUR");
            }

            return costeMaterial;
        }

        public decimal CalcularCosteMaterial(string idArticulo, string tipoCoste, string estructura, DateTime? fecha, bool usaDetalle, decimal? porcentajeMerma)
        {
            
            decimal coste = 0;
            string costeKey = String.Format("{0}_{1}_{2}_{3:yyyyMMdd}_{4}_{5:#0.00}", idArticulo, tipoCoste, estructura, fecha ?? DateTime.Now, usaDetalle, porcentajeMerma??0.05M);

            if (cacheCoste.ContainsKey(costeKey))
            {
                coste = cacheCoste[costeKey];
            }
            else
            { 
                var api = new ArticuloSharedAPI();
                string warnings = null;
                short idPlanta = api.ObtenerPlanta(idArticulo);
                coste = api.CosteMateriales(idArticulo, idPlanta, tipoCoste, estructura, fecha, usaDetalle, porcentajeMerma, out warnings);
                if (coste > 0)
                {
                    cacheCoste.Add(costeKey, coste);
                }
            }

            return coste;
        }

        public decimal? CalcularVA(decimal? precio, decimal? coste, bool cantidadVA)
        {
            decimal? valorVA = null;
            if (precio.HasValue && coste.HasValue && coste.Value > 0 && precio.Value >0)
            {
                if (cantidadVA)
                {
                    valorVA = precio.Value - coste.Value;
                }
                else
                {
                    valorVA = 100 * (precio.Value - coste.Value) / precio.Value;
                }
            }

            return valorVA;
        }
    }
}
