﻿using IQAP.Ofertas.Negocio.API;
using IQAP.Ofertas.Negocio.Modelo;
using IQAP.Shared.Common.Data;
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Data.Entity;

namespace IQAP.Ofertas.Negocio.Datos
{
    public class CalculoRepositorio : Repositorio<Calculo>
    {
       
        public CalculoRepositorio(OfertasEntities context) : base(context)
        {
        }

        public OfertasEntities contextoOferta
        {
            get { return Contexto as OfertasEntities; }
        }

        public List<DatosCalculo> SeleccionParametrizaciones(short? idNegocio)
        {
            return (from n in contextoOferta.Negocios join c in contextoOferta.Calculos on n.IdNegocio equals c.IdNegocio into leftJoin
                    from calc in leftJoin.DefaultIfEmpty()
                    where n.IdNegocio == (idNegocio.HasValue ? idNegocio : n.IdNegocio)
                    select new DatosCalculo()
                    {
                        IdCalculo = (short)(calc != null ? calc.IdCalculo : 0),
                        IdNegocio = n.IdNegocio,
                        Negocio = n.Descripcion,
                        Descripcion = calc.Descripcion,
                        EscaladoCostes = contextoOferta.Calculo_EscaladoCoste.Where(c => c.IdCalculo == calc.IdCalculo).Select(s=> s.Descripcion).ToList(),
                        CriterioAjusteEscaladoKg = contextoOferta.Calculo_Criterio.Where(c=> c.IdCalculo == calc.IdCalculo && c.IdCriterio == (short) Criterios.AjusteEscaladoKg).Count()>0,
                        CriterioArea = contextoOferta.Calculo_Criterio.Where(c => c.IdCalculo == calc.IdCalculo && c.IdCriterio == (short)Criterios.Area).Count() > 0,
                        CriterioPolimero = contextoOferta.Calculo_Criterio.Where(c => c.IdCalculo == calc.IdCalculo && c.IdCriterio == (short)Criterios.Polimero).Count() > 0,
                        DescuentoDistribuidor = calc.DescuentoDistribuidor,
                        Comision = calc.Comision
                    }).ToList();
        }

        internal List<Criterio> SeleccionarCriteriosPendientes(short idCalculo)
        {
            var listCriteriosAnadidos = contextoOferta.Calculo_Criterio.Where(c => c.IdCalculo == idCalculo).Select(c => c.IdCriterio).ToList();
            listCriteriosAnadidos.Add((short)Criterios.ValoresCalculo);

            return contextoOferta.Criterios.Where(c => !listCriteriosAnadidos.Contains(c.IdCriterio)).ToList();
        }

        public List<Calculo_Criterio> SeleccionarCriterios(short idCalculo)
        {
            return contextoOferta.Calculo_Criterio.Where(c => c.IdCalculo == idCalculo).OrderBy(c=> c.IdCriterio).ToList();
        }

        public DataTable SeleccionarValoresCriterioBase(short idCalculo, short idNegocio, int? idEscaladoCoste, short? idAplicacion, short? idEscaladoKg, short idTipoPrecio)
        {
            DataTable dt = new DataTable("Resultados");
            
            using (var cmd = (Contexto.Database.Connection as SqlConnection).CreateCommand())
            {
                cmd.CommandText = "SP_Calculo_ObtenerValoresCriterioBase";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@idCalculo", SqlDbType.SmallInt).Value = idCalculo;
                cmd.Parameters.Add("@idNegocio", SqlDbType.SmallInt).Value = idNegocio;
                cmd.Parameters.Add("@idEscaladoCoste", SqlDbType.Int).Value = idEscaladoCoste ?? 0;
                cmd.Parameters.Add("@idAplicacion", SqlDbType.SmallInt).Value = idAplicacion ?? 0;
                cmd.Parameters.Add("@idEscaladoKg", SqlDbType.SmallInt).Value = idEscaladoKg ?? 0;
                cmd.Parameters.Add("@idTipoPrecio", SqlDbType.SmallInt).Value = idTipoPrecio;

                using (var da = new SqlDataAdapter(cmd))
                {
                    da.Fill(dt);
                    return dt;
                }
            }
        }

        public DataTable SeleccionarValoresCriterioAjusteKg(short idCalculo, short idNegocio, short idTipoPrecio)
        {
            DataTable dt = new DataTable("Resultados");

            using (var cmd = (Contexto.Database.Connection as SqlConnection).CreateCommand())
            {
                cmd.CommandText = "SP_Calculo_ObtenerValoresAjusteKg";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@idCalculo", SqlDbType.SmallInt).Value = idCalculo;
                cmd.Parameters.Add("@idNegocio", SqlDbType.SmallInt).Value = idNegocio;
                cmd.Parameters.Add("@idTipoPrecio", SqlDbType.SmallInt).Value = idTipoPrecio;

                using (var da = new SqlDataAdapter(cmd))
                {
                    da.Fill(dt);
                    return dt;
                }
            }
        }

        public List<CriterioBaseEscaladoPrecio> ObtenerEscaladosPreciosCriterioAjusteKg(short idCalculo)
        {
            return contextoOferta.Calculo_EscaladoKg.Where(a => a.IdCalculo == idCalculo).Select(v=> new CriterioBaseEscaladoPrecio() { IdEscaladoKg = v.IdEscaladoKg, IdTipoPrecio = v.IdTipoPrecio }).Distinct().ToList();
        }

        internal bool UsaCriterio(short idNegocio, short idCriterio)
        {
            return contextoOferta.Calculo_Criterio.Where(c => c.Calculo.IdNegocio == idNegocio).Any(c=> c.IdCriterio == idCriterio);
        }

        public DataTable SeleccionarValoresCriterioZona(short idCalculo, short idTipoPrecio)
        {
            DataTable dt = new DataTable("Resultados");

            using (var cmd = (Contexto.Database.Connection as SqlConnection).CreateCommand())
            {
                cmd.CommandText = "SP_Calculo_ObtenerValoresZona";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@idCalculo", SqlDbType.SmallInt).Value = idCalculo;
                cmd.Parameters.Add("@idTipoPrecio", SqlDbType.SmallInt).Value = idTipoPrecio;

                using (var da = new SqlDataAdapter(cmd))
                {
                    da.Fill(dt);
                    return dt;
                }
            }
        }
        public DataTable SeleccionarValoresCriterioPolimero(short idCalculo, short idTipoPrecio)
        {
            DataTable dt = new DataTable("Resultados");

            using (var cmd = (Contexto.Database.Connection as SqlConnection).CreateCommand())
            {
                cmd.CommandText = "SP_Calculo_ObtenerValoresPolimero";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@idCalculo", SqlDbType.SmallInt).Value = idCalculo;
                cmd.Parameters.Add("@idTipoPrecio", SqlDbType.SmallInt).Value = idTipoPrecio;

                using (var da = new SqlDataAdapter(cmd))
                {
                    da.Fill(dt);
                    return dt;
                }
            }
        }
        public void AnadirEscaladoCoste(Calculo_EscaladoCoste escaladoCoste)
        {
            contextoOferta.Calculo_EscaladoCoste.Add(escaladoCoste);
        }

        public void AnadirCriterio(short idCalculo, short idCriterio)
        {
            var entity = new Calculo_Criterio() { IdCalculo = idCalculo, IdCriterio = idCriterio };

            contextoOferta.Calculo_Criterio.Add(entity);
        }

        public void EliminarCriterio(short idCalculo, short idCriterio)
        {
            var criterioCalculo = contextoOferta.Calculo_Criterio.Where(c => c.IdCalculo == idCalculo && c.IdCriterio == idCriterio).FirstOrDefault();

            if(criterioCalculo!=null)
            { 
                contextoOferta.Calculo_Criterio.Remove(criterioCalculo);
            }

        }

        public Calculo ObtenerCalculoNegocio(short idNegocio)
        {
            Calculo calculo = contextoOferta.Calculos.Include(c => c.Criterios).Where(c => c.IdNegocio == idNegocio).FirstOrDefault();

            return calculo;
        }

        public ConfiguracionCalculo ObtenerConfiguracionCalculo(short idNegocio, short idAplicacion, string idArea, short idPolimero)
        {
            ConfiguracionCalculo config = new ConfiguracionCalculo();

            config.Calculo = contextoOferta.Calculos.Include(c=> c.Criterios).Include(c=> c.EscaladosCoste).Where(c => c.IdNegocio == idNegocio).FirstOrDefault();

            if (config.Calculo != null)
            {
                config.ValoresBase = contextoOferta.Calculo_Valores.Where(v => v.IdCalculo == config.Calculo.IdCalculo && v.IdAplicacion == idAplicacion).ToList();

                //Criterio 2
                if (config.ContieneCriterio((short)Criterios.AjusteEscaladoKg))
                {
                    config.ValoresAjusteEscaladoKg = contextoOferta.Calculo_EscaladoKg.Where(a => a.IdCalculo == config.Calculo.IdCalculo && a.IdAplicacion == idAplicacion).ToList();
                }

                //Criterio 3
                if (config.ContieneCriterio((short)Criterios.Area))
                {
                    config.ValoresAreas = contextoOferta.Calculo_Area.Where(a => a.IdCalculo == config.Calculo.IdCalculo && a.IdArea == idArea).ToList();
                }

                //Criterio 4
                if (config.ContieneCriterio((short)Criterios.Polimero))
                {
                    config.ValoresPolimeros = contextoOferta.Calculo_Polimero.Where(f => f.IdCalculo == config.Calculo.IdCalculo && f.IdPolimero == idPolimero).ToList();
                }

                //Criterio 5 y 6 no requieren cargar más datos
            }
            return config;
        }

        public Dictionary<short, decimal> ObtenerTarifaCatalogo(string idArticulo, string moneda)
        {
            Dictionary<short, decimal> precios = new Dictionary<short, decimal>();
            using (SqlConnection cn = Contexto.Database.Connection as SqlConnection)
            {
                var cmd = cn.CreateCommand();
                cmd.CommandText = "SP_InterfaseM3_PrecioCatalogo";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@articulo", SqlDbType.VarChar).Value = idArticulo;
                cmd.Parameters.Add("@moneda", SqlDbType.VarChar).Value = moneda ?? string.Empty;

                cn.Open();
                var dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    short escalado = Convert.ToInt16(dr["EscaladoKg"]);
                    decimal valor = Convert.ToDecimal(dr["Valor"]);
                    if (escalado > 0)
                    {

                        precios.Add(escalado, valor);
                    }
                }
                dr.Close();

            }

            return precios;
        }

        internal DataTable ObtenerHistoricoPreciosCliente(string idArticulo, string idCliente, string moneda)
        {
            DataTable dt = new DataTable();
            using (SqlConnection cn = Contexto.Database.Connection as SqlConnection)
            {
                var cmd = cn.CreateCommand();
                cmd.CommandText = "[SP_InterfaseM3_ObtenerPreciosCliente]";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@articulo", SqlDbType.VarChar).Value = idArticulo;
                cmd.Parameters.Add("@cliente", SqlDbType.VarChar).Value = idCliente;
                cmd.Parameters.Add("@moneda", SqlDbType.VarChar).Value = moneda ?? string.Empty;

                cn.Open();
                using (var da = new SqlDataAdapter(cmd))
                {
                    da.Fill(dt);
                    return dt;
                }
            }
        }
    

        internal DataTable ObtenerHistoricoPreciosOtrosCliente(string idArticulo, string idCliente, string moneda)
        {
            DataTable dt = new DataTable();
            using (SqlConnection cn = Contexto.Database.Connection as SqlConnection)
            {
                var cmd = cn.CreateCommand();
                cmd.CommandText = "[SP_InterfaseM3_ObtenerPreciosOtrosClientes]";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@articulo", SqlDbType.VarChar).Value = idArticulo;
                cmd.Parameters.Add("@cliente", SqlDbType.VarChar).Value = idCliente;
                cmd.Parameters.Add("@moneda", SqlDbType.VarChar).Value = moneda ?? string.Empty;

                cn.Open();
                using (var da = new SqlDataAdapter(cmd))
                {
                    da.Fill(dt);
                    return dt;
                }
            }
        }

        private List<Precio> FillPrecios(SqlDataReader dr)
        {
            var precios = new List<Precio>();

            short columnaInicioEscaladosKg = 8;
            List<string> escaladosKg = Enumerable.Range(columnaInicioEscaladosKg, dr.FieldCount - columnaInicioEscaladosKg).Select(dr.GetName).ToList();

            while (dr.Read())
            {
                var precio = new Precio();

                precio.Orden = Convert.ToInt16(dr["Orden"]);
                precio.EsVigente = Convert.ToBoolean(dr["EsVigente"]);
                precio.TipoTarifa = dr["TipoTarifa"].ToString();
                precio.OrigenPrecio = dr["OrigenPrecio"].ToString();
                precio.FechaInicio = Convert.ToDateTime(dr["FechaInicio"]);
                precio.FechaFin = Convert.ToDateTime(dr["FechaFin"]);
                precio.Cliente = dr["Cliente"].ToString();
                precio.Moneda = dr["Moneda"].ToString();

                foreach(var idEscaladoKgString in escaladosKg)
                {
                    short idEscaladoKg;
                    if (short.TryParse(idEscaladoKgString, out idEscaladoKg))
                    {
                        var valor = DbHelper.GetNullableDecimal(dr[idEscaladoKgString]);
                        if (valor.HasValue)
                        {
                            precio.PreciosEscalado.Add(idEscaladoKg, valor.Value);
                        }
                    }
                }

                precios.Add(precio);
            }

            return precios;
        }
    }
}
