﻿using IQAP.Ofertas.Negocio.Modelo;
using System;
using System.Collections.Generic;
using System.Linq;
using IQAP.Shared.Common.Data;
using System.Data.Entity;
using System.Data.SqlClient;
using System.Data;

namespace IQAP.Ofertas.Negocio.Datos
{
    public class OfertaRepositorio : Repositorio<Oferta>
    {       
        public OfertaRepositorio(OfertasEntities context) : base(context)
        {
        }

        public OfertasEntities ContextoOferta
        {
            get { return Contexto as OfertasEntities; }
        }

        internal void IntegrarOfertaM3(int idOferta)
        {
            using (var cn = new SqlConnection(Contexto.Database.Connection.ConnectionString))
            {
                using (var cmd = cn.CreateCommand())
                { 
                    cmd.CommandText = "SP_InterfaseM3_GuardarOferta";
                    cmd.CommandTimeout = 60000;
                    cmd.CommandType = CommandType.StoredProcedure;

                    cmd.Parameters.Add("@idOferta", SqlDbType.Int).Value = idOferta;
                    cn.Open();
                    cmd.ExecuteNonQuery();
                }
            }
        }

        public override void Anadir(Oferta entity)
        {
            ContextoOferta.Ofertas.Add(entity);
            if(entity.Negocio!=null)
            {
                ContextoOferta.Entry<Negocio.Modelo.Negocio>(entity.Negocio).State = EntityState.Unchanged;
            }
        }

        public List<DatosOferta> FiltrarOfertas(int negocio, DateTime fechaDesde, DateTime fechaHasta, string descripcion, string representante, string cliente, string asistente, string articulo, bool ofertasImpresas, bool ofertasHistoricas)
        {

            List<DatosOferta> ofertas = new List<DatosOferta>();

            using (SqlConnection cn = Contexto.Database.Connection as SqlConnection)
            {
                var cmd = cn.CreateCommand();
                cmd.CommandText = "SP_ConsultaOfertas";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@negocio", SqlDbType.Int).Value = negocio;
                cmd.Parameters.Add("@fechaDesde", SqlDbType.DateTime).Value = fechaDesde.Date;
                cmd.Parameters.Add("@fechaHasta", SqlDbType.DateTime).Value = fechaHasta.Date;
                cmd.Parameters.Add("@descripcion", SqlDbType.VarChar, 100).Value = descripcion;
                cmd.Parameters.Add("@representante", SqlDbType.VarChar, 15).Value = representante;
                cmd.Parameters.Add("@cliente", SqlDbType.VarChar, 15).Value = cliente;
                cmd.Parameters.Add("@asistente", SqlDbType.VarChar, 15).Value = asistente;
                cmd.Parameters.Add("@articulo", SqlDbType.VarChar, 15).Value = articulo;
                cmd.Parameters.Add("@impresa", SqlDbType.Bit).Value = ofertasImpresas;
                cmd.Parameters.Add("@historicas", SqlDbType.Bit).Value = ofertasHistoricas;

                cn.Open();
                var dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    DatosOferta oferta = new DatosOferta();

                    oferta.IdOferta = Convert.ToInt32(dr["IdOferta"]);
                    oferta.IdOfertaOrigen = dr["IdOfertaOrigen"] != DBNull.Value ? Convert.ToInt32(dr["IdOfertaOrigen"]) : (int?) null;
                    oferta.NumeroVersion = Convert.ToInt32(dr["NumeroVersion"]);
                    oferta.DescripcionOferta = dr["DescripcionOferta"] != DBNull.Value ? dr["DescripcionOferta"].ToString() : String.Empty;
                    oferta.Negocio = dr["DescripcionNegocio"] != DBNull.Value ? dr["DescripcionNegocio"].ToString() : String.Empty;
                    oferta.IdCliente = dr["IdCliente"].ToString();
                    oferta.DescripcionCliente = dr["NombreCliente"] != DBNull.Value ? dr["NombreCliente"].ToString() : String.Empty;
                    oferta.Fecha = Convert.ToDateTime(dr["Fecha"]);
                    oferta.FechaImpresion = dr["FechaImpresion"] != DBNull.Value ? Convert.ToDateTime(dr["FechaImpresion"]): (DateTime?) null;
                    oferta.DescripcionRepresentante = dr["NombreRepresentante"] != DBNull.Value ? dr["NombreRepresentante"].ToString() : String.Empty;
                    oferta.DescripcionResponsable = dr["NombreResponsable"] != DBNull.Value ? dr["NombreResponsable"].ToString() : String.Empty;
                    oferta.DescripcionAsistente = dr["NombreAsistente"] != DBNull.Value ? dr["NombreAsistente"].ToString() : String.Empty;
                    oferta.Articulos = dr["Articulos"] != DBNull.Value ? dr["Articulos"].ToString() : String.Empty;
                    oferta.IntegradaM3 = Convert.ToBoolean(dr["IntegradaM3"].ToString());

                    ofertas.Add(oferta);
                }

                dr.Close();
            }

            return ofertas;
        }

        internal void GuardarHistorico(int idOferta)
        {
            var ofertaHistorico = ContextoOferta.Ofertas.Where(o => o.IdOferta == idOferta).Include(o => o.OfertaLineas.Select(l => l.OfertaLineaPrecios)).FirstOrDefault();

            ofertaHistorico.IdOfertaOrigen = idOferta;
            ofertaHistorico.FechaCreacion = DateTime.Now;
            ofertaHistorico.UsuarioCreacion = System.Environment.UserName;
            ofertaHistorico.FechaModificacion = DateTime.Now;
            ofertaHistorico.UsuarioModificacion = System.Environment.UserName;

            ContextoOferta.Ofertas.Add(ofertaHistorico);

            //Marcar la lineas como added también.
            foreach (var linea in ofertaHistorico.OfertaLineas)
            {
                ContextoOferta.Entry<OfertaLinea>(linea).State = EntityState.Added;
                foreach (var precio in linea.OfertaLineaPrecios)
                {
                    ContextoOferta.Entry<OfertaLineaPrecio>(precio).State = EntityState.Added;
                }
            }

        }

        public List<DatosOferta> FiltrarArticuloOfertadoCliente(string idArticulo, string idCliente)
        {
            List<DatosOferta> ofertas = new List<DatosOferta>();

            using (SqlConnection cn = Contexto.Database.Connection as SqlConnection)
            {
                var cmd = cn.CreateCommand();
                cmd.CommandText = "SP_ConsultaArticuloOfertadoCliente";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@IdArticulo", SqlDbType.VarChar).Value = idArticulo;
                cmd.Parameters.Add("@IdCliente", SqlDbType.VarChar).Value = idCliente;

                cn.Open();
                var dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    DatosOferta oferta = new DatosOferta();

                    oferta.IdOferta = Convert.ToInt32(dr["IdOferta"]);
                    oferta.DescripcionOferta = dr["Descripcion"] != null ? dr["Descripcion"].ToString() : String.Empty;
                    oferta.IdCliente = dr["IdCliente"] != null ? dr["IdCliente"].ToString() : String.Empty;
                    oferta.Fecha = Convert.ToDateTime(dr["FechaCreacion"]);
                    oferta.Articulos = dr["IdArticulo"] != null ? dr["IdArticulo"].ToString() : String.Empty;

                    ofertas.Add(oferta);
                }
                dr.Close();
            }
            return ofertas;
        }

        internal void EliminarLinea(int idOfertaLinea)
        {
            var linea = ContextoOferta.OfertaLineas.Include(l=> l.OfertaLineaPrecios).Include(l=>l.OfertaLineaPrecioCalculados).Where(l=>l.IdOfertaLinea == idOfertaLinea).SingleOrDefault();
            if (linea != null)
            {
                ContextoOferta.OfertaLineaPrecios.RemoveRange(linea.OfertaLineaPrecios);
                ContextoOferta.OfertaLineaPrecioCalculadoes.RemoveRange(linea.OfertaLineaPrecioCalculados);
                ContextoOferta.OfertaLineas.Remove(linea);
            }
        }

        internal OfertaLinea ObtenerLinea(int idOfertaLinea)
        {
            return ContextoOferta.OfertaLineas
                                    .Include(l=> l.Oferta.Negocio)
                                    .Include(l=> l.OfertaLineaPrecios)
                                    .Include(l=>l.OfertaLineaPrecioCalculados)
                                .Where(l=> l.IdOfertaLinea == idOfertaLinea).FirstOrDefault();
        }

        internal OfertaLineaPrecio ObtenerLineaPrecio(int idOfertaLinea, short idEscaladoKg)
        {
            return ContextoOferta.OfertaLineaPrecios.Where(p => p.IdOfertaLinea == idOfertaLinea && p.IdEscaladoKg == idEscaladoKg).FirstOrDefault();
        }

        internal IEnumerable<OfertaLinea> SeleccionarLineas(Func<OfertaLinea, bool> predicate)
        {
            return ContextoOferta.OfertaLineas.Include(l=>l.OfertaLineaPrecios).Where(predicate).ToList();
        }

        internal void ModificarLinea(OfertaLinea linea)
        {
            //Añadir nuevos precios
            ContextoOferta.OfertaLineaPrecios.RemoveRange(ContextoOferta.OfertaLineaPrecios.Where(p => p.IdOfertaLinea == linea.IdOfertaLinea).ToList());
            if (linea.OfertaLineaPrecios!=null)
            {
               ContextoOferta.OfertaLineaPrecios.AddRange(linea.OfertaLineaPrecios);
            }

            //Añadir nuevos precios calculados
            ContextoOferta.OfertaLineaPrecioCalculadoes.RemoveRange(ContextoOferta.OfertaLineaPrecioCalculadoes.Where(p => p.IdOfertaLinea == linea.IdOfertaLinea).ToList());
            if (linea.OfertaLineaPrecioCalculados != null)
            {
                ContextoOferta.OfertaLineaPrecioCalculadoes.AddRange(linea.OfertaLineaPrecioCalculados);
            }

            //Modificar entidad principal
            ContextoOferta.OfertaLineas.Attach(linea);
            ContextoOferta.Entry<OfertaLinea>(linea).State = EntityState.Modified;
        }

        internal List<int> SeleccionarLineasCambiosVersionAnterior(int idOferta)
        {
            var ofertaAnterior = ContextoOferta.Ofertas.Where(o => o.IdOfertaOrigen == idOferta).OrderByDescending(o => o.NumeroVersion).FirstOrDefault();
            if(ofertaAnterior!=null)
            { 
                return ContextoOferta.OfertaLineas.Where(l => l.IdOferta == idOferta && l.FechaModificacion > ofertaAnterior.FechaModificacion).Select(l => l.IdOfertaLinea).ToList();
            }

            return new List<int>();
        }

        internal void CambioDivisa(int idOferta, decimal factorConversion)
        {
            var lineaOferta = ContextoOferta.OfertaLineas.Include(l => l.OfertaLineaPrecios).Include(l => l.OfertaLineaPrecioCalculados).Where(l => l.IdOferta == idOferta).ToList();

            lineaOferta.ForEach(l => {
                //Precio único
                if(l.EsPrecioUnico)
                {
                    l.PrecioUnico = l.PrecioUnico.Value * factorConversion;
                }
                //Precios
                l.OfertaLineaPrecios.ToList().ForEach(p => p.Valor = p.Valor * factorConversion);
                //Precios calculados
                l.OfertaLineaPrecioCalculados.ToList().ForEach(p => p.Valor = p.Valor * factorConversion);
                //Importes estimados
                l.ImporteEstimado = l.ImporteEstimado * factorConversion;
                l.VAEstimado = l.VAEstimado * factorConversion;
                l.CosteMaterial = l.CosteMaterial * factorConversion;
            });
        }

        internal List<PrecioEscaladoTarifa> ObtenerPreciosTarifaLineas(int idOferta)
        {
            List<PrecioEscaladoTarifa> precios = new List<PrecioEscaladoTarifa>();

            using (SqlConnection cn = Contexto.Database.Connection as SqlConnection)
            {
                var cmd = cn.CreateCommand();
                cmd.CommandText = "SP_InterfaseM3_ObtenerPreciosTarifa";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@idOferta", SqlDbType.Int).Value = idOferta;

                cn.Open();
                var dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    PrecioEscaladoTarifa precioTarifa = new PrecioEscaladoTarifa();

                    precioTarifa.Articulo = dr["Articulo"].ToString();
                    precioTarifa.EscaladoKg = Convert.ToInt16(dr["EscaladoKg"]);
                    precioTarifa.Valor = Convert.ToDecimal(dr["Valor"]);

                    precios.Add(precioTarifa);
                }
                dr.Close();
            }
            return precios;
        }

        internal void AnadirLinea(OfertaLinea linea)
        {
            ContextoOferta.OfertaLineas.Add(linea);

            //No añadir oferta relacionada y negocio 
            if (linea.Oferta != null)
            {
                ContextoOferta.Entry<Oferta>(linea.Oferta).State = EntityState.Unchanged;
                if (linea.Oferta.Negocio != null)
                {
                    ContextoOferta.Entry<Negocio.Modelo.Negocio>(linea.Oferta.Negocio).State = EntityState.Unchanged;
                }
            }
            
        }
        public void EliminarOferta(int idOferta)
        {
            var oferta = ContextoOferta.Ofertas
                .Include(o => o.OfertaLineas.Select(l => l.OfertaLineaPrecios))
                .Include(o => o.OfertaLineas.Select(l => l.OfertaLineaPrecioCalculados))
                .Include(o=> o.ClientesExcluidos)
                .Where(o => o.IdOferta==idOferta).FirstOrDefault();

            if (oferta != null)
            {
                //Eliminar Precios
                foreach(var l in oferta.OfertaLineas)
                {
                    ContextoOferta.OfertaLineaPrecioCalculadoes.RemoveRange(l.OfertaLineaPrecioCalculados.ToList());
                    ContextoOferta.OfertaLineaPrecios.RemoveRange(l.OfertaLineaPrecios.ToList());
                }

                //Eliminar lineas
                ContextoOferta.OfertaLineas.RemoveRange(oferta.OfertaLineas.ToList());
                //Eliminar clientes excluidos
                ContextoOferta.OfertaClienteExcluidoes.RemoveRange(oferta.ClientesExcluidos.ToList());
                //Eliminar oferta
                ContextoOferta.Ofertas.Remove(oferta);
            }
        }

        internal bool ValidarFechasVigencia(DateTime fechaInicio, DateTime fechaFin, string idCliente, string moneda, bool esSPOT)
        {
            bool existeOfertaConFechaFinDiferente = false;
            using (SqlConnection cn = Contexto.Database.Connection as SqlConnection)
            {
                var cmd = cn.CreateCommand();
                cmd.CommandText = "SP_InterfaseM3_ValidarFechasVigencia";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@fechaInicio", SqlDbType.DateTime).Value = fechaInicio;
                cmd.Parameters.Add("@fechaFin", SqlDbType.DateTime).Value = fechaFin;
                cmd.Parameters.Add("@idCliente", SqlDbType.VarChar, 15).Value = idCliente;
                cmd.Parameters.Add("@moneda", SqlDbType.VarChar).Value = moneda ?? string.Empty;
                cmd.Parameters.Add("@esSPOT", SqlDbType.Bit).Value = esSPOT ? 1 : 0;

                cn.Open();
                var dr = cmd.ExecuteReader();
                if (dr.Read())
                {
                    existeOfertaConFechaFinDiferente = true;
                }
                dr.Close();
            }

            return !existeOfertaConFechaFinDiferente;
        }

        public override void Modificar(Oferta entity)
        {
            if(entity.OfertaLineas!=null)
            {
                entity.OfertaLineas = null;
            }
            ContextoOferta.Ofertas.Attach(entity);
            ContextoOferta.Entry<Oferta>(entity).State = EntityState.Modified;
        }

        internal void AnadirLineaPrecio(OfertaLineaPrecio entity)
        {
            ContextoOferta.OfertaLineaPrecios.Add(entity);
        }

        internal void EliminarLineaPrecio(OfertaLineaPrecio entity)
        {
            ContextoOferta.OfertaLineaPrecios.Remove(entity);
        }
    }
}
