﻿using IQAP.Ofertas.Negocio.Modelo;
using IQAP.Shared.Common.Data;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IQAP.Ofertas.Negocio.Datos
{
    public interface IRepositorioManager : IDisposable
    {
        int SaveChanges();
    }

    public class RepositorioManager : IRepositorioManager
    {
        private readonly OfertasEntities Contexto;

        public OfertaRepositorio Ofertas { get; private set; }
        public IRepositorio<OfertaClienteExcluido> OfertaClienteExcluidos { get; private set; }
        public IRepositorio<Modelo.Negocio> Negocios { get; private set; }
        public IRepositorio<Representante> Representantes { get; private set; }
        public IRepositorio<ClienteFinalTarifa> ClientesFinales { get; private set; }
        public CalculoRepositorio Calculos { get; private set; }
        public IRepositorio<EscaladoKg> EscaladosKg { get; private set; }
        public AplicacionRepositorio Aplicaciones { get; private set; }
        public IRepositorio<Polimero> Polimeros { get; private set; }
        public IRepositorio<Proceso> Procesos { get; private set; }
        public IRepositorio<Modelo.Area> Areas { get; private set; }
        public IRepositorio<FormaPago> FormasPago { get; private set; }
        public IRepositorio<CondicionEntrega> CondicionesEntrega { get; private set; }

        public IRepositorio<Calculo_Valores> CalculoValores { get; private set; }
        public IRepositorio<Modelo.Calculo_EscaladoCoste> Calculo_EscaladoCostes { get; private set; }
        public IRepositorio<Calculo_EscaladoKg> CalculoEscaladoKg { get; private set; }
        public IRepositorio<Calculo_Area> CalculoArea { get; private set; }
        public IRepositorio<Calculo_Polimero> CalculoPolimero { get; private set; }
      

         public RepositorioManager(OfertasEntities context)
        {
            Contexto = context;
            Ofertas = new OfertaRepositorio(context);
            OfertaClienteExcluidos = new Repositorio<OfertaClienteExcluido>(context);
            Negocios = new Repositorio<Modelo.Negocio>(context);
            Representantes = new Repositorio<Representante>(context);
            ClientesFinales = new Repositorio<ClienteFinalTarifa>(context);
            Areas = new Repositorio<Area>(context);
            Calculos = new CalculoRepositorio(context);
            EscaladosKg = new Repositorio<EscaladoKg>(context);
            Aplicaciones = new AplicacionRepositorio(context);
            Polimeros = new Repositorio<Polimero>(context);
            Procesos = new Repositorio<Proceso>(context);
            Calculo_EscaladoCostes = new Repositorio<Modelo.Calculo_EscaladoCoste>(context);
            CalculoValores = new Repositorio<Calculo_Valores>(context);
            CalculoEscaladoKg = new Repositorio<Calculo_EscaladoKg>(context);
            CalculoArea = new Repositorio<Calculo_Area>(context);
            CalculoPolimero = new Repositorio<Calculo_Polimero>(context);
            FormasPago = new Repositorio<FormaPago>(context);
            CondicionesEntrega = new Repositorio<CondicionEntrega>(context);
        }


        public void Dispose()
        {
            Contexto.Dispose();
        }

        public int SaveChanges()
        {
            return Contexto.SaveChanges();
        }
    }
}

