﻿using IQAP.Ofertas.Negocio.Modelo;
using IQAP.Shared.Common.Data;
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IQAP.Ofertas.Negocio.Datos
{
    public class TarifaDataProvider: IDisposable
    {
        private readonly OfertasEntities Contexto;
        public TarifaDataProvider(OfertasEntities context)
        {
            Contexto = context;
        }

        internal DataTable ConsultarTarifas(FiltroConsultaTarifas filtro)
        {
            var ds = new DataSet();

            using (SqlConnection cn = Contexto.Database.Connection as SqlConnection)
            {
                var cmd = cn.CreateCommand();
                cmd.CommandText = "SP_InterfaseM3_ConsultaTarifas";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@grupoCliente", SqlDbType.VarChar, 15).Value = filtro.Grupo;
                cmd.Parameters.Add("@cliente", SqlDbType.VarChar, 15).Value = filtro.Cliente;
                cmd.Parameters.Add("@clienteFinal", SqlDbType.VarChar, 15).Value = filtro.ClienteFinal;
                cmd.Parameters.Add("@representante", SqlDbType.VarChar, 15).Value = filtro.Representante;
                cmd.Parameters.Add("@articulo", SqlDbType.VarChar, 15).Value = filtro.Articulo;
                cmd.Parameters.Add("@tipoTarifa", SqlDbType.VarChar, 2).Value = filtro.ListaPrecios;
                cmd.Parameters.Add("@fecha", SqlDbType.DateTime).Value = filtro.Fecha.HasValue ? filtro.Fecha.Value : (object)DBNull.Value;
                cmd.Parameters.Add("@verHistorico", SqlDbType.Bit).Value = filtro.VerHistorico;
                cmd.Parameters.Add("@verCoste", SqlDbType.Bit).Value = filtro.VerCoste;
                cmd.Parameters.Add("@verReferenciaInterna", SqlDbType.Bit).Value = filtro.VerReferenciaInterna;
                cmd.Parameters.Add("@listaAsistentes", SqlDbType.VarChar, 200).Value = String.Join(",", filtro.AsistentesPermitidas.ToArray());

                cn.Open();

                using (var da = new SqlDataAdapter(cmd))
                {
                    da.Fill(ds);
                    return ds.Tables[0];
                }
            }
        }

        internal DataTable ConsultarComparativaVA(FiltroConsultaComparativaVA filtro)
        {
            var ds = new DataSet();

            using (SqlConnection cn = Contexto.Database.Connection as SqlConnection)
            {
                var cmd = cn.CreateCommand();
                cmd.CommandText = "SP_InterfaseM3_ConsultaComparativaVA";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@grupoCliente", SqlDbType.VarChar, 15).Value = filtro.Grupo;
                cmd.Parameters.Add("@cliente", SqlDbType.VarChar, 15).Value = filtro.Cliente;
                cmd.Parameters.Add("@clienteFinal", SqlDbType.VarChar, 15).Value = filtro.ClienteFinal;
                cmd.Parameters.Add("@articulo", SqlDbType.VarChar, 15).Value = filtro.Articulo;
                cmd.Parameters.Add("@verReferenciaInterna", SqlDbType.Bit).Value = filtro.VerReferenciaInterna;
                cmd.Parameters.Add("@escaladoKg", SqlDbType.Int).Value = filtro.EscaladoKg.ToADOParameterValue(true);

                cmd.Parameters.Add("@listaAsistentes", SqlDbType.VarChar, 200).Value = String.Join(",", filtro.AsistentesPermitidas.ToArray());

                cn.Open();

                using (var da = new SqlDataAdapter(cmd))
                {
                    da.Fill(ds);
                    return ds.Tables[0];
                }
            }
        }

        internal void GuardarIncrementarPrecios(IncrementoPrecio simulacion)
        {
            Contexto.IncrementosPrecios.Add(simulacion);
            Contexto.SaveChanges();   
        }

        internal void IncrementarPrecios(int idIncrementoPrecios)
        {
            using (var cn = Contexto.Database.Connection as SqlConnection)
            {
                using (var cmd = cn.CreateCommand())
                {
                    cmd.CommandText = "SP_InterfaseM3_IncrementarPrecios";
                    cmd.CommandTimeout = 60000;
                    cmd.CommandType = CommandType.StoredProcedure;

                    cmd.Parameters.Add("@idIncrementoPrecios", SqlDbType.Int).Value = idIncrementoPrecios;
                    cn.Open();
                    cmd.ExecuteNonQuery();
                }
            }
        }
        public void Dispose()
        {
            Contexto.Dispose();
        }

        internal DataTable ObtenerTarifasFuturas(string cliente, string moneda, DateTime fechaInicio)
        {
            var ds = new DataSet();

            using (SqlConnection cn = Contexto.Database.Connection as SqlConnection)
            {
                var cmd = cn.CreateCommand();
                cmd.CommandText = "SP_InterfaseM3_ExisteTarifaFutura";
                cmd.CommandTimeout = 60000;
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@cliente", SqlDbType.VarChar, 15).Value = cliente;
                cmd.Parameters.Add("@moneda", SqlDbType.VarChar).Value = moneda ?? string.Empty;
                cmd.Parameters.Add("@fechaInicio", SqlDbType.DateTime).Value = fechaInicio;

                cn.Open();

                using (var da = new SqlDataAdapter(cmd))
                {
                    da.Fill(ds);
                    return ds.Tables[0];
                }
            }
        }

        internal bool ExistenTarifasIncrementoPrecios(string cliente, DateTime fechaInicio, DateTime fechaFin)
        {
            return Contexto.IncrementosPrecios.Any(i => i.Cliente == cliente && i.NuevaFechaInicio >= fechaInicio && i.NuevaFechaInicio < fechaFin);
        }
    }
}
