﻿using IQAP.Ofertas.Negocio.API;
using IQAP.Ofertas.Negocio.Datos;
using IQAP.Ofertas.Negocio.Modelo;
using IQAP.Ofertas.Recursos.Textos;
using IQAP.Shared.Common;
using IQAP.Shared.Negocio;
using IQAP.Shared.Negocio.API;
using IQAP.Shared.Negocio.Modelo;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


namespace IQAP.Ofertas.Negocio.Informes
{
    public enum Informes
    {
        DocOferta = 1,
        DocTarifas = 2,
        DocIncrementoPrecios = 3
    }
    public class InformeManager
    {
        private const int Max_EscaladosKg = 11;
        public void ImprimirOferta(Oferta oferta, List<string> articulosImprimir, Cliente cliente, Cliente clienteGrupo, FormaPago formaPago, CondicionEntrega condicionesEntrega, string filename)
        {
            Logger.Info(String.Format("Inicio Impresión oferta: {0}", oferta.IdOferta));
            //Rellenar datos
            var gtr = new GestorTextosReport();
            string escaladoPrecioUnicoDescripcion = gtr.ObtenerTextoDocumento((short) Informes.DocOferta, "PrecioUnico", oferta.Idioma);
            var ds = FillOfertaDataset(oferta, articulosImprimir, cliente, clienteGrupo, formaPago, condicionesEntrega, escaladoPrecioUnicoDescripcion);

            //Render pdf

            using (crDocOferta informe = new crDocOferta())
            {
                informe.SetDataSource(ds);
                informe.Refresh();

                //Obtener parametros de traduccion textos
                var textos = gtr.ObtenerTextosDocumento((short) Informes.DocOferta, oferta.Idioma);

                foreach (var param in textos)
                {
                    informe.SetParameterValue(param.Key, param.Value);
                }
               
                informe.ExportToDisk(CrystalDecisions.Shared.ExportFormatType.PortableDocFormat, filename);

                Logger.Info(String.Format("Fin Impresión oferta: {0}", oferta.IdOferta));
            }
        }

        public void ImprimirTarifas(DataTable dt, int columnaPrimerEscaladoKg, FiltroConsultaTarifas filtro, string filename, bool verCabecera, bool agruparClienteFinal, string idioma)
        {
            var gtr = new GestorTextosReport();

            //Rellenar datos
            var ds = FillTarifaDataset(dt, columnaPrimerEscaladoKg, filtro, verCabecera, agruparClienteFinal);

            //Render pdf
            using (var informe = new crDocTarifas())
            {
                informe.SetDataSource(ds);
                informe.Refresh();

                //Obtener parametros de traduccion textos
                var textos = gtr.ObtenerTextosDocumento((short)Informes.DocTarifas, idioma);

                foreach (var param in textos)
                {
                    informe.SetParameterValue(param.Key, param.Value);
                }

                informe.ExportToDisk(CrystalDecisions.Shared.ExportFormatType.PortableDocFormat, filename);
            }
        }

        public void ImprimirIncrementoPrecios(DataTable dt, Cliente cliente, int columnaPrimerEscaladoKg, string filename, string idioma)
        {
            var gtr = new GestorTextosReport();

            //Rellenar datos
            var ds = FillIncrementoPreciosDataset(dt, columnaPrimerEscaladoKg);

            //Render pdf
            using (var informe = new crDocIncrementoPrecios())
            {
                informe.SetDataSource(ds);
                informe.Refresh();

                //Obtener parametros de traduccion textos
                var textos = gtr.ObtenerTextosDocumento((short)Informes.DocIncrementoPrecios, idioma);

                foreach (var param in textos)
                {
                    informe.SetParameterValue(param.Key, param.Value);
                }

                informe.ExportToDisk(CrystalDecisions.Shared.ExportFormatType.PortableDocFormat, filename);
            }
        }

        private dsIncrementoPrecios FillIncrementoPreciosDataset(DataTable dt, int columnaPrimerEscaladoKg)
        {
            var ds = new dsIncrementoPrecios();

            var sapi = new ArticuloSharedAPI();

            //Lista de escalados no vacios
            List<int> ListaIndicesEscaladosValidos = ObtenerListaIndicesEscaladosValidos(dt, columnaPrimerEscaladoKg);


            var informeRow = ds.Informe.NewInformeRow();
            informeRow.IdInforme = 1;
            
            //Escalados 
            for (int i = 0; i < Math.Min(Max_EscaladosKg, ListaIndicesEscaladosValidos.Count); i++)
            {
                decimal escaladoKg = 0;
                if (decimal.TryParse(dt.Columns[ListaIndicesEscaladosValidos[i]].ColumnName, out escaladoKg))
                {
                    informeRow["EscaladoKg" + (i + 1).ToString()] = string.Format("{0:#,##0.##}", escaladoKg);
                }
                else
                {
                    informeRow["EscaladoKg" + (i + 1).ToString()] = dt.Columns[ListaIndicesEscaladosValidos[i]].ColumnName;
                }
            }

            ds.Informe.AddInformeRow(informeRow);

            //Lineas
            foreach (DataRow row in dt.Rows)
            {
                var tarifaRow = ds.Tarifa.NewTarifaRow();

                tarifaRow.GrupoCliente = row.Field<string>("GrupoCliente");
                tarifaRow.GrupoClienteDescripcion = row.Field<string>("GrupoClienteDescripcion");
                tarifaRow.Cliente = row.Field<string>("Cliente");
                tarifaRow.ClienteDescripcion = row.Field<string>("ClienteDescripcion");
                tarifaRow.ClienteFinal = row.Field<string>("ClienteFinal");
                tarifaRow.ClienteFinalDescripcion = row.Field<string>("ClienteFinalDescripcion");
                tarifaRow.Articulo = sapi.CodigoArticuloAlbaran(row.Field<string>("IdArticulo"));
                tarifaRow.ArticuloDescripcion = row.Field<string>("ArticuloDescripcion");
                tarifaRow.Moneda = row.Field<string>("Moneda");
                tarifaRow.FechaInicio = row.Field<DateTime>("FechaInicio");
                tarifaRow.FechaFin = row.Field<DateTime>("FechaFin");

                //Escalados 
                for (int i = 0; i < Math.Min(Max_EscaladosKg, ListaIndicesEscaladosValidos.Count); i++)
                {
                    tarifaRow["Precio" + (i + 1).ToString()] = row.Field<decimal?>(ListaIndicesEscaladosValidos[i]) ?? (object)DBNull.Value;
                }

                ds.Tarifa.AddTarifaRow(tarifaRow);
            }

            return ds;
        }

        private dsTarifas FillTarifaDataset(DataTable dt, int columnaPrimerEscaladoKg, FiltroConsultaTarifas filtro, bool verCabecera, bool agruparClienteFinal)
        {
            var ds = new dsTarifas();

            var sapi = new ArticuloSharedAPI();

            //Lista de escalados no vacios
            List<int> ListaIndicesEscaladosValidos = ObtenerListaIndicesEscaladosValidos(dt, columnaPrimerEscaladoKg);


            var informeRow = ds.Informe.NewInformeRow();
            informeRow.IdInforme = 1;
            informeRow.GrupoCliente = filtro.Grupo;
            if(!String.IsNullOrEmpty(filtro.Grupo))
            {
                informeRow.GrupoClienteDescripcion = dt.AsEnumerable().Select(r => r.Field<string>("GrupoClienteDescripcion")).FirstOrDefault();
            }
            informeRow.Cliente = filtro.Cliente;
            if (!String.IsNullOrEmpty(filtro.Cliente))
            {
                informeRow.ClienteDescripcion = dt.AsEnumerable().Select(r => r.Field<string>("ClienteDescripcion")).FirstOrDefault();
            }
            informeRow.ClienteFinal = filtro.ClienteFinal;
            if (!String.IsNullOrEmpty(filtro.ClienteFinal))
            {
                informeRow.ClienteFinalDescripcion = dt.AsEnumerable().Select(r => r.Field<string>("ClienteFinalDescripcion")).FirstOrDefault();
            }
            informeRow.ListaPrecio = filtro.ListaPrecios;
            informeRow.Articulo = filtro.Articulo;
            if (!String.IsNullOrEmpty(filtro.Articulo))
            {
                informeRow.ArticuloDescripcion = dt.AsEnumerable().Select(r => r.Field<string>("ArticuloDescripcion")).FirstOrDefault();
            }
            if (filtro.Fecha.HasValue)
            {
                informeRow.Fecha = filtro.Fecha.Value;
            }
            informeRow.VerHistorico = filtro.VerHistorico;
            informeRow.VerReferenciaInterna = filtro.VerReferenciaInterna;
            informeRow.VerCoste = filtro.VerCoste;
            informeRow.VerCabeceraInforme = verCabecera;
            informeRow.AgruparClienteFinal = agruparClienteFinal;

            //Escalados 
            for (int i = 0; i < Math.Min(Max_EscaladosKg, ListaIndicesEscaladosValidos.Count); i++)
            {
                decimal escaladoKg = 0;
                if (decimal.TryParse(dt.Columns[ListaIndicesEscaladosValidos[i]].ColumnName, out escaladoKg))
                {
                    informeRow["EscaladoKg" + (i + 1).ToString()] = string.Format("{0:#,##0.##}", escaladoKg);
                }
                else
                {
                    informeRow["EscaladoKg" + (i + 1).ToString()] = dt.Columns[ListaIndicesEscaladosValidos[i]].ColumnName;
                }
            }

            ds.Informe.AddInformeRow(informeRow);

            //Lineas
            foreach (DataRow row in dt.Rows)
            {
                var tarifaRow = ds.Tarifa.NewTarifaRow();

                tarifaRow.GrupoCliente = row.Field<string>("GrupoCliente");
                tarifaRow.GrupoClienteDescripcion = row.Field<string>("GrupoClienteDescripcion");
                tarifaRow.Cliente = row.Field<string>("Cliente");
                tarifaRow.ClienteDescripcion = row.Field<string>("ClienteDescripcion");
                tarifaRow.ClienteFinal = row.Field<string>("ClienteFinal");
                tarifaRow.ClienteFinalDescripcion = row.Field<string>("ClienteFinalDescripcion");
                tarifaRow.Articulo = sapi.CodigoArticuloAlbaran(row.Field<string>("IdArticulo"));
                tarifaRow.ArticuloInterno = row.Field<string>("ArticuloInterno");
                tarifaRow.ArticuloDescripcion = row.Field<string>("ArticuloDescripcion");
                tarifaRow.Moneda = row.Field<string>("Moneda");
                tarifaRow.TipoTarifa = row.Field<string>("TipoTarifa");
                var coste = row.Field<decimal?>("Coste");
                if(coste.HasValue)
                {
                    tarifaRow.Coste = coste.Value;
                }
                tarifaRow.FechaInicio = row.Field<DateTime>("FechaInicio");
                tarifaRow.FechaFin = row.Field<DateTime>("FechaFin");

                //Escalados 
                for (int i = 0; i < Math.Min(Max_EscaladosKg, ListaIndicesEscaladosValidos.Count); i++)
                {
                    tarifaRow["Precio" + (i + 1).ToString()] = row.Field<decimal?>(ListaIndicesEscaladosValidos[i]) ?? (object)DBNull.Value;
                }

                tarifaRow.NoAplica = row.Field<bool>("NoAplica");

                ds.Tarifa.AddTarifaRow(tarifaRow);
            }

            return ds;
        }

        private List<int> ObtenerListaIndicesEscaladosValidos(DataTable dt, int columnaPrimerEscaladoKg)
        {
            var listaIndices = new List<int>();
            for (int i = columnaPrimerEscaladoKg; i < dt.Columns.Count; i++)
            {
                if (dt.AsEnumerable().Any(p =>p.Field<decimal?>(i).HasValue))
                {
                    listaIndices.Add(i);
                }
            }

            return listaIndices;
        }

        private dsOferta FillOfertaDataset(Oferta oferta, List<string> articulosImprimir, Cliente cliente, Cliente clienteGrupo, FormaPago formaPago, CondicionEntrega condicionesEntrega, string descripcionPrecioUnico)
        {
            var ds = new dsOferta();
            
            //Lista de articulos a imprimir
            var listaLineas = oferta.OfertaLineas;
            if (articulosImprimir != null && articulosImprimir.Count > 0)
            {
                listaLineas = listaLineas.Where(l => articulosImprimir.Contains(l.IdArticulo)).ToList();
            }

            List<short> listaEscalados = oferta.ObtenerListaEscaladosKg(listaLineas.ToList());
            string representante = cliente.Representante;

            if(oferta.IdRepresentante != cliente.IdRepresentante)
            {
                var rapi = new RepresentanteAPI();
                var rep = rapi.ObtenerRepresentante(oferta.IdRepresentante);
                if(rep!=null)
                {
                    representante = rep.Nombre;
                }
            }

            //Rellenar ds con oferta info
            var ofertaRow = ds.Oferta.NewOfertaRow();
            ofertaRow.IdOferta = oferta.IdOferta;
            ofertaRow.MesesValidez = oferta.IndicarFechasVigencia? -1 : oferta.MesesValidez;
            ofertaRow.ClienteNombre = cliente.Nombre;
            ofertaRow.ClienteContacto = oferta.Contacto;
            ofertaRow.ClienteTelefono = oferta.ContactoTelefono;
            ofertaRow.ClienteFax = oferta.ContactoFax;
            ofertaRow.ClienteEmail = oferta.ContactoEmail;
            ofertaRow.Observaciones = oferta.Observaciones ?? string.Empty;
            ofertaRow.Autor = oferta.Autor;
            ofertaRow.AutorModificado = !String.IsNullOrEmpty(oferta.AutorModificacion) && oferta.AutorModificacion != oferta.Autor? oferta.AutorModificacion : string.Empty;
            ofertaRow.Representante = representante;
            ofertaRow.OcultarRepresentante = oferta.OcultarRepresentanteDocumentoOferta;
            ofertaRow.Asistente = cliente.Asistente;
            ofertaRow.FormaPago = formaPago != null ? formaPago.Descripcion : string.Empty;
            ofertaRow.CondicionEntrega = condicionesEntrega != null ? condicionesEntrega.DescripcionCompleta : string.Empty;
            ofertaRow.DescripcionCondicionesEntrega = oferta.DescripcionCondicionesEntrega;
            ofertaRow.EsSPOT = oferta.EsOfertaSPOT;
            ofertaRow.FechaInicioVigencia = oferta.FechaInicioVigencia;
            ofertaRow.FechaFinVigencia = oferta.FechaFinVigencia ?? oferta.FechaFinVigenciaEstimada;
            ofertaRow.NumeroVersion = oferta.NumeroVersion;
            ofertaRow.Moneda = oferta.IdMoneda;

            if (oferta.EsOfertaGrupoCliente && clienteGrupo != null)
            {
                //Obtener clientes del grupo
                var api = new ClienteSharedAPI();

                var listaClientes = api.SeleccionarClientesGrupo(clienteGrupo.IdCliente);
                ofertaRow.EsOfertaClienteGrupo = true;
                ofertaRow.HayClientesGrupoExcluidos = oferta.ClientesExcluidos.Count > 0;
                ofertaRow.ListaClientesGrupo = String.Join("\n", listaClientes.Where(c => c.IdCliente != cliente.IdCliente
                                                    && !oferta.ClientesExcluidos.Select(e => e.IdCliente).Contains(c.IdCliente))
                                                    .Select(c=> c.Nombre).OrderBy(c=>c).ToArray());
            }
            

            //Escalados validos
            for (int i = 0; i < listaEscalados.Count; i++)
            {
                ofertaRow["EscaladoKg" + (i + 1).ToString()] = listaEscalados.ElementAt(i) == ListaEscaladosKg.EscaladoPrecioUnico ? descripcionPrecioUnico : string.Format("{0:#,###}", listaEscalados.ElementAt(i));
            }

            ds.Oferta.AddOfertaRow(ofertaRow);

            foreach (var linea in listaLineas)
            {
                
                var row = ds.OfertaLinea.NewOfertaLineaRow();
                var sapi = new ArticuloSharedAPI();
                //Referencia albarán
                row.Referencia = sapi.CodigoArticuloAlbaran(linea.IdArticulo);

                //Guardar descripcion del articulo en la oferta. Depende del color
                string descripcionColor = linea.Descripcion;
                string codigoArticuloInterno = linea.IdArticulo;
                string grupoTecnologico = linea.GrupoTecnologico;
                string subgrupoTecnologico = string.Empty;
                string grupoProducto = linea.GrupoProducto;
               
                var articulo = sapi.ObtenerArticulo(linea.IdArticulo);
                if (articulo != null)
                {
                    if(articulo.KIT)
                    {
                        codigoArticuloInterno = sapi.CodigoArticuloInterno(articulo.IdArticulo);
                    }
                    descripcionColor = articulo.Color;
                    grupoTecnologico = articulo.GrupoTecnologico;
                    grupoProducto = articulo.GrupoProducto;
                    subgrupoTecnologico = articulo.Clasificacion;
                }
                row.Descripcion = sapi.NombreComercial(codigoArticuloInterno, grupoTecnologico, subgrupoTecnologico, grupoProducto, descripcionColor);
                row.DescripcionAdicional = linea.DescripcionAdicional;

                //Precios de los escalados válidos
                for(int i =0; i< listaEscalados.Count;i++)
                {
                    var precio = linea.PrecioKg(listaEscalados.ElementAt(i));
                    if (precio.HasValue)
                    {
                        row["Precio" + (i+1).ToString()] = precio;
                    }
                }
                row.IdOfertaLinea = linea.IdOfertaLinea;
                row.IdOferta = oferta.IdOferta;

                ds.OfertaLinea.AddOfertaLineaRow(row);
            }

            return ds;
        }

        
    }
}
