﻿using IQAP.Ofertas.Negocio.API;
using IQAP.Ofertas.Negocio.Datos;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IQAP.Ofertas.Negocio.Modelo
{
    public class DatosPrecioCalculado
    {
        public Dictionary<short, decimal> Recomendado { get; set; }
        public Dictionary<short, decimal> Minimo { get; set; }
        public Dictionary<short, decimal> Coste { get; set; }

        public bool EsPrecioCatalogo { get; set; }
        public decimal? CosteMaterial { get; set; }

        public string WarningsCosteMaterial { get; set; }

        public DateTime? FechaCalculo { get; set; }

        public DatosPrecioCalculado(decimal costeMateriales)
        {
            CosteMaterial = costeMateriales;
            Coste = new Dictionary<short, decimal>();
            AsignarCostes(costeMateriales);
            Recomendado = new Dictionary<short, decimal>();
            Minimo = new Dictionary<short, decimal>();
        }

        private void AsignarCostes(decimal costeMateriales)
        {
            ListaEscaladosKg.EscaladosKg.ForEach(e => { AsignarCoste(e, costeMateriales);});
        }

        public void AsignarPrecioMinimo(short idEscaladoKg, decimal? valor)
        {
            AsignarPrecio(Minimo, idEscaladoKg, valor);
        }

        public void AsignarPrecioRecomendado(short idEscaladoKg, decimal? valor)
        {
            AsignarPrecio(Recomendado, idEscaladoKg, valor);
        }

        public void AsignarCoste(short idEscaladoKg, decimal? valor)
        {
            AsignarPrecio(Coste, idEscaladoKg, valor);
        }


        private void AsignarPrecio(Dictionary<short,decimal> precios, short idEscaladoKg, decimal? valor)
        {
            if(precios.ContainsKey(idEscaladoKg))
            {
                if (valor.HasValue)
                {
                    precios[idEscaladoKg] = valor.Value;
                }
                else
                {
                    precios.Remove(idEscaladoKg);
                }
            }
            else
            {
                if (valor.HasValue) precios.Add(idEscaladoKg, valor.Value);
            }
        }

        public decimal? ObtenerPrecio(short idEscaladoKg, short idTipoPrecio)
        {
            switch (idTipoPrecio)
            {
                case (short)Precios.Minimo:
                    return ObtenerPrecio(Minimo, idEscaladoKg);

                case (short)Precios.Recomendado:
                    return ObtenerPrecio(Recomendado, idEscaladoKg);
                default:
                    return null;
            }
        }

        public decimal? ObtenerPrecio(Dictionary<short, decimal> precios, short idEscaladoKg)
        {
            if(precios!=null && precios.ContainsKey(idEscaladoKg))
            {
                return precios[idEscaladoKg];
            }

            return null;
        }

        public string ObtenerPrecioString(short idEscaladoKg, short idTipoPrecio, string formato)
        {
            decimal? precio = ObtenerPrecio(idEscaladoKg, idTipoPrecio);

            return precio.HasValue ? precio.Value.ToString(formato) : string.Empty;
        }

        public void AsignarPrecio(short escaladoKg, decimal? valor, short idTipoPrecio)
        {
            if (idTipoPrecio == (short)Precios.Recomendado)
            {
                AsignarPrecioRecomendado(escaladoKg, valor);
            }
            if (idTipoPrecio == (short)Precios.Minimo)
            {
                AsignarPrecioMinimo(escaladoKg, valor);
            }
        }

        public void AplicarIncrementoPorcentaje(decimal? porcentajeIncremento, short idTipoPrecio)
        {
            if(porcentajeIncremento.HasValue && porcentajeIncremento!=0)
            {
                foreach(short escaladoKg in ListaEscaladosKg.EscaladosKg)
                {
                    decimal? valor = ObtenerPrecio(escaladoKg, idTipoPrecio);
                    if(valor.HasValue)
                    {
                        valor = valor + (valor * porcentajeIncremento / 100);
                        AsignarPrecio(escaladoKg, valor, idTipoPrecio);
                    }
                }
            }
        }
    }
}
