﻿using IQAP.Ofertas.Negocio.Datos;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IQAP.Ofertas.Negocio.Modelo
{
    public partial class OfertaLinea
    {
        public decimal? PrecioKg5 { get { return PrecioKg(5); } set { SetPrecioKg(5, value); }}
        public decimal? PrecioKg10 { get { return PrecioKg(10); } set { SetPrecioKg(10, value); }}
        public decimal? PrecioKg15 { get { return PrecioKg(15); } set { SetPrecioKg(15, value); }}

        public decimal? PrecioKg25 { get { return PrecioKg(25); } set { SetPrecioKg(25, value); } }
        public decimal? PrecioKg50 { get { return PrecioKg(50); } set { SetPrecioKg(50, value); } }
        public decimal? PrecioKg100 { get { return PrecioKg(100); } set { SetPrecioKg(100, value); } }
        public decimal? PrecioKg200 { get { return PrecioKg(200); } set { SetPrecioKg(200, value); } }
        public decimal? PrecioKg500 { get { return PrecioKg(500); } set { SetPrecioKg(500, value); } }
        public decimal? PrecioKg1000 { get { return PrecioKg(1000); } set { SetPrecioKg(1000, value); } }
        public decimal? PrecioKg2000 { get { return PrecioKg(2000); } set { SetPrecioKg(2000, value); } }
        public decimal? PrecioKg5000 { get { return PrecioKg(5000); } set { SetPrecioKg(5000, value); } }
        public decimal? PrecioKg10000 { get { return PrecioKg(10000); } set { SetPrecioKg(10000, value); } }
        public decimal? PrecioKg20000 { get { return PrecioKg(20000); } set { SetPrecioKg(20000, value); } }

        public decimal? PrecioKg(short idEscaladoKg)
        {
            if(idEscaladoKg == ListaEscaladosKg.EscaladoPrecioUnico)
            {
                return PrecioUnico;
            }

            if (this.OfertaLineaPrecios != null && this.OfertaLineaPrecios.Where(p => p.IdEscaladoKg == idEscaladoKg).Count() > 0)
            {
                return this.OfertaLineaPrecios.Where(p => p.IdEscaladoKg == idEscaladoKg).Select(p => p.Valor).FirstOrDefault();
            }

            return null;
        }

        public void SetPrecioKg(short idEscaladoKg, decimal? valor)
        {
            if (idEscaladoKg == ListaEscaladosKg.EscaladoPrecioUnico)
            {
                PrecioUnico = valor;
                return;
            }

            if (this.OfertaLineaPrecios == null)
            {
                this.OfertaLineaPrecios = new List<OfertaLineaPrecio>();
            }

            var precio = this.OfertaLineaPrecios.Where(p => p.IdEscaladoKg == idEscaladoKg).FirstOrDefault();
            if (precio!=null)
            {
                if(valor.HasValue)
                {
                    precio.Valor = valor.Value;
                }
                else
                {
                    this.OfertaLineaPrecios.Remove(precio);
                }
            }
            else
            {
                if (valor.HasValue)
                {
                    this.OfertaLineaPrecios.Add(new OfertaLineaPrecio() { IdEscaladoKg = idEscaladoKg, Valor = valor.Value });
                }
            }
        }

        public List<short> EscaladosKgLinea
        {
            get
            {
                if(this.EsPrecioUnico)
                {
                    return new List<short>() { ListaEscaladosKg.EscaladoPrecioUnico };
                }

                if (this.OfertaLineaPrecios != null && this.OfertaLineaPrecios.Count > 0)
                {
                    return this.OfertaLineaPrecios.Select(l => l.IdEscaladoKg).OrderBy(l => l).ToList();
                }
                return new List<short>();
            }
        }

        public short? MinimoEscaladosKgInformado
        {
            get
            {
                if (this.OfertaLineaPrecios != null && this.OfertaLineaPrecios.Count > 0)
                {
                    return OfertaLineaPrecios.Min(p => p.IdEscaladoKg);
                }

                return null;
            }
        }

        public bool EsPrecioUnico
        {
            get
            {
                return PrecioUnico.HasValue && PrecioUnico.Value > 0;
            }
        }

        public decimal? VAEstimadoPorcentaje
        {
            get
            {
                if (ImporteEstimado.HasValue && VAEstimado.HasValue && ImporteEstimado.Value >0)
                {
                    return 100 * VAEstimado.Value / ImporteEstimado.Value;
                }

                return null;
            }
        }

        public decimal? ImporteEstimadoCalculado
        {
            get
            {
                decimal? precio = ObtenerPrecio();
                if(precio.HasValue && CantidadEstimada.HasValue)
                {
                    return CantidadEstimada.Value * precio.Value;
                }
                return null;
            }
        }

        public decimal? VAEstimadoCalculado
        {
            get
            {
                decimal? precio = ObtenerPrecio();
                if (precio.HasValue && CantidadEstimada.HasValue && CosteMaterial.HasValue && CosteMaterial.Value>0)
                {
                    return CantidadEstimada.Value * (precio.Value - CosteMaterial.Value);
                }

                return null;
            }
        }

        private decimal? ObtenerPrecio()
        {
            if (CantidadEstimada.HasValue)
            {
                if (EsPrecioUnico)
                {
                    return PrecioUnico;
                }
                else
                {
                    if (OfertaLineaPrecios != null)
                    {
                        foreach (var idEscaladoKg in ListaEscaladosKg.EscaladosKgPulverizados.Where(e => CantidadEstimada.Value >= e).OrderByDescending(e => e))
                        {
                            var precio = OfertaLineaPrecios.Where(p => p.IdEscaladoKg == idEscaladoKg).FirstOrDefault();
                            if (precio != null)
                            {
                                return precio.Valor;
                            }
                        }
                    }
                }
            }

            return null;
        }
    }
}
