﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using IQAP.Ofertas.Negocio;
using IQAP.Ofertas.Negocio.Modelo;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using Newtonsoft.Json;
using IQAP.Ofertas.Services.Models;

namespace IQAP.Ofertas.Service.Tests
{
    [TestClass]
    public class CalculadoraTest
    {
        [TestMethod]
        public void Post_referencia_con_HttpClient()
        {
            //Result Previsto
            List<KeyValuePair<short, decimal>> esperado = new List<KeyValuePair<short, decimal>>();
            esperado.Add(new KeyValuePair<short, decimal>(25, (decimal)13.070515));
            esperado.Add(new KeyValuePair<short, decimal>(50, (decimal)8.580515));
            esperado.Add(new KeyValuePair<short, decimal>(100, (decimal)5.750515));
            esperado.Add(new KeyValuePair<short, decimal>(200, (decimal)4.920515));
            esperado.Add(new KeyValuePair<short, decimal>(500, (decimal)4.240515));
            esperado.Add(new KeyValuePair<short, decimal>(1000, (decimal)3.730515));
            esperado.Add(new KeyValuePair<short, decimal>(2000, (decimal)3.380515));
            esperado.Add(new KeyValuePair<short, decimal>(5000, (decimal)3.100515));
            esperado.Add(new KeyValuePair<short, decimal>(10000, (decimal)2.950515));
            esperado.Add(new KeyValuePair<short, decimal>(20000, (decimal)2.850515));

            //Arrange
            var parametros = CrearParametro();

            //Act
            var task = RunPostReferenciaHttpClientAsync(parametros);
            task.Wait();
            var result = (List<KeyValuePair<short, decimal>>)task.Result;
            //Assert
            Assert.IsTrue(esperado == result, "Los precios son validos");
        }

        private ParametrosCalculo CrearParametro()
        {
            var parametro = new ParametrosCalculo();
            parametro.IdArticulo = "700111";
            parametro.IdAplicacion = 2;
            parametro.IdPolimero = 1;
            //parametro.IdPlanta = 7;
            parametro.IdNegocio = 1;
            parametro.Moneda = "EUR";
            parametro.MonedaValorCambio = 1;

            return parametro;
        }

        static async Task<List<KeyValuePair<short, decimal>>> RunPostReferenciaHttpClientAsync(ParametrosCalculo parametros)
        {
            var result = new List<KeyValuePair<short, decimal>>();

            using (var client = GetHttpClient())
            {
                // HTTP POST               
                var param = new StringContent(JsonConvert.SerializeObject(parametros), Encoding.UTF8, "application/json");
                var response = await client.PostAsync("OfertasServices/Calculadora/Calcular", param);

                if (response.IsSuccessStatusCode)
                {
                    var value = await response.Content.ReadAsStringAsync();

                    result = JsonConvert.DeserializeObject<List<KeyValuePair<short, decimal>>>(value);
                }
            }

            return result;
        }

        public static HttpClient GetHttpClient()
        {
            var handler = new HttpClientHandler()
            {
                Credentials = new NetworkCredential("webservice", "iqap")
            };

            var client = new HttpClient(handler)
            {
                BaseAddress = new Uri("https://app.iqapgroup.com")
            };

            client.DefaultRequestHeaders.Accept.Clear();
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

            return client;
        }
    }
}
