﻿using IQAP.Ofertas.Negocio;
using IQAP.Ofertas.Negocio.API;
using IQAP.Ofertas.Negocio.Modelo;
using IQAP.Ofertas.Services.Models;
using IQAP.Shared.Common;
using IQAP.Shared.Negocio.API;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;

namespace IQAP.Ofertas.Services.Controllers
{
    [RoutePrefix("Calculadora")]
    public class CalculadoraController : ApiController
    {
        //Torna el resultat de la calculadora
        [Route("Calcular")]
        [HttpPost]
        public IHttpActionResult Calcular([FromBody] ParametrosCalculo parametros)

        {
            try
            {
                //Creo un objeto para pasar los parámetros que espera la API
                ParametrosCalculoPrecio parametro = new ParametrosCalculoPrecio();
                parametro.IdArticulo = parametros.IdArticulo != null ? parametros.IdArticulo : String.Empty;
                parametro.IdAplicacion = parametros.IdAplicacion;
                parametro.IdNegocio = parametros.IdNegocio;
                parametro.IdPolimero = parametros.IdPolimero;
                parametro.Moneda = parametros.Moneda;
                parametro.MonedaValorCambio = parametros.MonedaValorCambio;

                var api = new CalculoAPI();
                DatosPrecioCalculado PrecioCalculado;

                if (!String.IsNullOrEmpty(parametro.IdArticulo))
                {
                    //Calcular la planta a partir del articulo
                    var sapi = new ArticuloSharedAPI();
                    parametro.IdPlanta = sapi.ObtenerPlanta(parametro.IdArticulo);

                    PrecioCalculado = api.CalcularPrecios(parametro);
                }
                else
                {
                    PrecioCalculado = api.CalcularPrecios(parametro, parametros.Coste, null);
                }


                if (PrecioCalculado != null)
                {
                    var listaRecomendados = PrecioCalculado.Recomendado.OrderBy(p => p.Key).ToList();
                    return Ok(listaRecomendados);
                }
                else
                {
                    return NotFound();
                }

            }
            catch (Exception ex)
            {
                //Aqui pondremos un control del error en un log.
                Logger.Error(String.Format("Calcular/{0}", parametros.IdArticulo), ex);
                return InternalServerError(ex);
            }
        }

        #region CargarCombos

        [Route("Negocios")]
        [HttpGet]
        public IHttpActionResult CargarNegocios()
        {
            try
            {
                var api = new NegocioAPI();
                var negocios = api.ObtenerNegocios();
                if (negocios != null)
                {
                    var lista = new List<Models.Negocio>();
                    foreach (Negocio.Modelo.Negocio n in negocios)
                    {
                        var negocio = new Models.Negocio();
                        negocio.IdNegocio = n.IdNegocio;
                        negocio.Descripcion = n.Descripcion;

                        lista.Add(negocio);
                    }
                    return Ok(lista);
                }
                else
                {
                    throw new HttpResponseException(HttpStatusCode.NotFound);
                }
            }
            catch (Exception ex)
            {
                //Aqui pondremos un control del error en un log.
                Logger.Error(String.Format("Cargando Negocios", ex));
                throw new HttpResponseException(HttpStatusCode.InternalServerError);
            }
        }

        [Route("Polimeros")]
        [HttpGet]
        public IHttpActionResult CargarPolimeros()
        {
            try
            {
                var api = new CalculoAPI();
                var polimeros = api.ObtenerPolimeros();
                if (polimeros != null)
                {
                    var lista = new List<Models.Polimero>();
                    foreach (Negocio.Modelo.Polimero p in polimeros)
                    {
                        var polimero = new Models.Polimero();
                        polimero.IdPolimero = p.IdPolimero;
                        polimero.Descripcion = p.Descripcion;
                        lista.Add(polimero);
                    }

                    return Ok(lista);
                }
                else
                {
                    throw new HttpResponseException(HttpStatusCode.NotFound);
                }
            }
            catch (Exception ex)
            {
                //Aqui pondremos un control del error en un log.
                Logger.Error(String.Format("Cargando Polimeros", ex));
                throw new HttpResponseException(HttpStatusCode.InternalServerError);
            }
        }
        
        [Route("Aplicaciones/{idNegocio}")]
        [HttpGet]
        public IHttpActionResult CargarAplicacinoes(short idNegocio)
        {
            try
            {
                var api = new CalculoAPI();
                var aplicaciones = api.ObtenerAplicaciones(idNegocio);
                if (aplicaciones != null)
                {
                    var lista = new List<Models.Aplicacion>();
                    foreach (Negocio.Modelo.Aplicacion a in aplicaciones)
                    {
                        var aplicacion = new Models.Aplicacion();
                        aplicacion.IdAplicacion = a.IdAplicacion;
                        aplicacion.IdNegocio = a.IdNegocio;
                        aplicacion.Descripcion = a.Descripcion;
                        
                        lista.Add(aplicacion);
                    }

                    return Ok(lista);                    
                }
                else
                {
                    throw new HttpResponseException(HttpStatusCode.NotFound);
                }
            }
            catch (Exception ex)
            {
                //Aqui pondremos un control del error en un log.
                Logger.Error(String.Format("Cargando Aplicaciones", ex));
                throw new HttpResponseException(HttpStatusCode.InternalServerError);
            }
        }

        [Route("Divisas")]
        [HttpGet]
        public IHttpActionResult CargarDivisas()
        {
            try
            {
                var api = new DivisaSharedAPI();
                var divisas = api.ObtenerDivisas();
                if (divisas != null)
                {
                    var lista = new List<Models.Moneda>();
                    foreach (Shared.Negocio.Modelo.Moneda m in divisas)
                    {
                        var moneda = new Models.Moneda();
                        moneda.Divisa = m.Divisa;
                        moneda.ValorCambio = m.ValorCambio;

                        lista.Add(moneda);
                    }

                    return Ok(lista);
                }
                else
                {
                    throw new HttpResponseException(HttpStatusCode.NotFound);
                }
            }
            catch (Exception ex)
            {
                //Aqui pondremos un control del error en un log.
                Logger.Error(String.Format("Cargando Divisas", ex));
                throw new HttpResponseException(HttpStatusCode.InternalServerError);
            }
        }

        #endregion

        #region Test
        [Route("Encurso")]
        [HttpGet]
        public IHttpActionResult FabricacionesEnCurso()
        {
            try
            {
                var parametro = new ParametrosCalculoPrecio();
                parametro.IdArticulo = "";
                parametro.IdAplicacion = 2;
                parametro.IdPolimero = 1;
                parametro.IdPlanta = 7;
                parametro.IdNegocio = 1;
                parametro.Moneda = "EUR";
                parametro.MonedaValorCambio = 1;

                var api = new CalculoAPI();
                var PrecioCalculado = api.CalcularPrecios(parametro, 1, null);
                if (PrecioCalculado != null)
                {
                    return Ok(PrecioCalculado);
                }
                else
                {
                    return NotFound();
                }

            }
            catch (Exception ex)
            {
                return InternalServerError(ex);
            }
        }
        #endregion
    }
}
