﻿using IQAP.Ofertas.Negocio;
using IQAP.Ofertas.Negocio.API;
using IQAP.Ofertas.Negocio.Modelo;
using IQAP.Ofertas.Services.Datos;
using IQAP.Ofertas.Services.Models;
using IQAP.Shared.Common;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;

namespace IQAP.Ofertas.Services.Controllers
{
    [RoutePrefix("Oferta")]
    public class OfertaController : ApiController
    {
        //Torna una llista de ofertes
        [Route("Ofertas")]
        [HttpPost]
        public IHttpActionResult Ofertas([FromBody] ParametrosOferta parametros)
        {
            var api = new OfertaAPI();
            var ofertas = api.SeleccionarOfertasWeb(parametros.Cliente, parametros.Articulo);
            //var ofertas = api.SeleccionarOfertas(0, DateTime.Now.AddYears(-10), DateTime.Now.AddYears(1), String.Empty, String.Empty , cliente, String.Empty, articulo, false, false);
            try
            {
                if (ofertas == null)
                {
                    return NotFound();
                }

                else
                {
                    return Ok(ofertas);

                }
            }
            catch (Exception ex)
            {
                Logger.Error(String.Format("Error Service Ofertas/{0}/{1}", parametros.Cliente, parametros.Articulo), ex);
                return InternalServerError(ex);
            }

        }

        //Torna una llista de linies d'una oferta
        [Route("Lineas/{idOferta}")]
        [HttpGet]
        public IHttpActionResult LineasOferta(int idOferta)
        {
            var api = new OfertaAPI();
            var lineas = api.SeleccionarLineasOfertaWeb(idOferta);
            
            try
            {
                if (lineas == null)
                {
                    return NotFound();
                }

                else
                {
                    return Ok(lineas);

                }
            }
            catch (Exception ex)
            {
                Logger.Error(String.Format("Error Service LineasOferta/{0}", idOferta), ex);
                return InternalServerError(ex);
            }
        }

        //Torna una llista de linies d'una oferta
        [Route("Precios/{idLineaOferta}")]
        [HttpGet]
        public IHttpActionResult PreciosLineaOferta(int idLineaOferta)
        {
            var api = new OfertaAPI();
            var lineas = api.ObtenerLineaOferta(idLineaOferta);

            try
            {
                if (lineas == null)
                {
                    return NotFound();
                }

                else
                {
                    
                    var listaRecomendados = lineas.OfertaLineaPrecios.ToDictionary(p=>p.IdEscaladoKg, p=>p.Valor).OrderBy(p => p.Key).ToList();
                    return Ok(listaRecomendados);                    

                }
            }
            catch (Exception ex)
            {
                Logger.Error(String.Format("Error Service PreciosLineaOferta/{0}", idLineaOferta), ex);
                return InternalServerError(ex);
            }
        }

        //Envia oferta per mail
        [Route("MailOferta")]
        [HttpPost]
        public IHttpActionResult EnviarMail([FromBody] EnvioMail mail)
        {
            try
            {
                var api = new EnviarMailAPI();
                
            
                if (api.Envio(mail))
                {
                    return Ok();
                }

                else
                {
                    return NotFound();

                }
            }
            catch (Exception ex)
            {
                Logger.Error(String.Format("Error Enviando ofertas {0} ", mail.IdOferta), ex);
                return InternalServerError(ex);
            }

        }
    }
}
