﻿using IQAP.OfertasAPP.Negocio.Model;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace IQAP.OfertasAPP.Negocio.Data
{
    internal class CalculadoraDataProvider
    {

        //Els el prefix per les crides a les parts del webservice        
        private const string resourcePrefix = "OfertasServices/Calculadora";

        //Retorna una llista amb l'escalat de preus
        internal async Task<List<KeyValuePair<short,decimal>>> GetPrecio(ParametrosCalculoPrecio parametro)
        {
            using (var client = HttpHelper.GetHttpClient())
            {
                // HTTP POST               
                var param = new StringContent(JsonConvert.SerializeObject(parametro), Encoding.UTF8, "application/json");
                var response = await client.PostAsync(resourcePrefix + "/Calcular", param);
                
                if (response.IsSuccessStatusCode)
                {
                    var value = await response.Content.ReadAsStringAsync();

                    return JsonConvert.DeserializeObject<List<KeyValuePair<short, decimal>>>(value);
                }
            }
            return null;
        }

        internal async Task<List<Model.Negocio>> GetNegocios()
        {
           using (var client = HttpHelper.GetHttpClient())
           {
                // HTTP GET  
                HttpResponseMessage response = await client.GetAsync(String.Format("{0}/Negocios",resourcePrefix));
                if (response.IsSuccessStatusCode)
                {
                    var data = await response.Content.ReadAsStringAsync();

                    return JsonConvert.DeserializeObject<List<Model.Negocio>>(data);
                }
            }
            return null;
        }

        internal async Task<List<Model.Polimero>> GetPolimeros()
        {
            using (var client = HttpHelper.GetHttpClient())
            {
                // HTTP GET  
                HttpResponseMessage response = await client.GetAsync(String.Format("{0}/Polimeros", resourcePrefix));
                if (response.IsSuccessStatusCode)
                {
                    var data = await response.Content.ReadAsStringAsync();

                    return JsonConvert.DeserializeObject<List<Model.Polimero>>(data);
                }
            }
            return null;
        }

        internal async Task<List<Model.Aplicacion>> GetAplicaciones(short idNegocio)
        {
            using (var client = HttpHelper.GetHttpClient())
            {
                // HTTP GET  
                HttpResponseMessage response = await client.GetAsync(String.Format("{0}/Aplicaciones/{1}", resourcePrefix, idNegocio));
                if (response.IsSuccessStatusCode)
                {
                    var data = await response.Content.ReadAsStringAsync();

                    return JsonConvert.DeserializeObject<List<Model.Aplicacion>>(data);
                }
            }
            return null;
        }

        internal async Task<List<Model.Moneda>> GetDivisas()
        {
            using (var client = HttpHelper.GetHttpClient())
            {
                // HTTP GET  
                HttpResponseMessage response = await client.GetAsync(String.Format("{0}/Divisas", resourcePrefix));
                if (response.IsSuccessStatusCode)
                {
                    var data = await response.Content.ReadAsStringAsync();

                    return JsonConvert.DeserializeObject<List<Model.Moneda>>(data);
                }
            }
            return null;
        }
    }
}
