﻿using IQAP.Ofertas.Services.Models;
using IQAP.OfertasAPP.Negocio.Model;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using System.Net;

namespace IQAP.OfertasAPP.Negocio.Data
{
    internal class OfertasDataProvider
    {
        //Els el prefix per les crides a les parts del webservice        
        private const string resourcePrefix = "OfertasServices/Oferta";

        //Retorna una llista amb totes les ofertes
        internal async Task<List<Oferta>> GetOfertas(ParametrosOferta parametro)
        {
           using (var client = HttpHelper.GetHttpClient())
           {
                // HTTP POST
                var response = await client.PostAsync(resourcePrefix + "/Ofertas", new StringContent(JsonConvert.SerializeObject(parametro), Encoding.UTF8, "application/json"));
                
                if (response.IsSuccessStatusCode)
                {                    
                    var value = await response.Content.ReadAsStringAsync();

                   return JsonConvert.DeserializeObject<List<Oferta>>(value);                    
                }
            }
            return null;
        }

        //Retorna una llista amb totes les línies de la oferta
        internal async Task<List<LineaOferta>> GetLineasOferta(int idOferta)
        {
            using (var client = HttpHelper.GetHttpClient())
            {
                // HTTP GET  
                HttpResponseMessage response = await client.GetAsync(String.Format("{0}/Lineas/{1}", resourcePrefix, idOferta));
                if (response.IsSuccessStatusCode)
                {
                    var data = await response.Content.ReadAsStringAsync();

                    return JsonConvert.DeserializeObject<List<LineaOferta>>(data);
                }
            }
            return null;
        }

        //Retorna una llista amb totes les línies de la oferta
        internal async Task<List<KeyValuePair<short, decimal>>> GetPreciosLineaOferta(int idLineaOferta)
        {
            using (var client = HttpHelper.GetHttpClient())
            {
                // HTTP GET  
                HttpResponseMessage response = await client.GetAsync(String.Format("{0}/Precios/{1}", resourcePrefix, idLineaOferta));
                if (response.IsSuccessStatusCode)
                {
                    var data = await response.Content.ReadAsStringAsync();

                    return JsonConvert.DeserializeObject<List<KeyValuePair<short, decimal>>>(data);
                }
            }
            return null;
        }

        //Envia la oferta per mail
        internal async Task<bool> EnviarMail(int idOferta, List<string> articulos, string destinatario)
        {
            using (var client = HttpHelper.GetHttpClient())
            {
                var parametro = new EnvioMail();
                parametro.IdOferta = idOferta;
                parametro.Articulos = articulos;
                parametro.Destinatario = destinatario;

                // HTTP POST  
                var response = await client.PostAsync(resourcePrefix + "/MailOferta", new StringContent(JsonConvert.SerializeObject(parametro), Encoding.UTF8, "application/json"));

                if (response.IsSuccessStatusCode)
                {
                    return true;
                }
                else return false;
            }            
        }
    }
}
