﻿using System;
using System.Globalization;
using System.Reflection;
using System.Resources;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace IQAP.OfertasApp.Helper
{
    // Classe per traduir els textos
    [ContentProperty("Text")]
    public class TranslateExtension : IMarkupExtension
    {
        readonly CultureInfo ci;
        const string ResourceId = "IQAP.OfertasApp.Recursos.Textos";

        static readonly Lazy<ResourceManager> ResMgr =
            new Lazy<ResourceManager>(() => new ResourceManager(
                ResourceId,
                typeof(TranslateExtension).GetTypeInfo().Assembly));

        public TranslateExtension()
        {
            if (Application.Current.Properties.ContainsKey("currentLanguage"))
            {
                ci = (CultureInfo)Application.Current.Properties["currentLanguage"];
            }
            else
            {
                ci = DependencyService.Get<ILocalize>().GetCurrentCultureInfo();
                Application.Current.Properties["currentLanguage"] = ci;
            }
        }

        public string Text { get; set; }

        public object ProvideValue(IServiceProvider serviceProvider)
        {
            if (Text == null)
            {
                return String.Empty;
            }

            var translation = ResMgr.Value.GetString(Text, ci);

            if (translation == null)
            {

                translation = Text; // returns the key, which GETS DISPLAYED TO THE USER

            }

            return translation;
        }
    }
}
