﻿using Acr.UserDialogs;
using IQAP.OfertasApp.Helper;
using IQAP.OfertasApp.Recursos;
using IQAP.OfertasApp.Views;
using IQAP.OfertasAPP.Negocio.API;
using IQAP.OfertasAPP.Negocio.Helpers;
using IQAP.OfertasAPP.Negocio.Model;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using Xamarin.Forms;

namespace IQAP.OfertasApp.ViewModels
{
    public class CalculadoraViewModel : BaseViewModel
    {
        #region Atributos      
        private string articulo;
        private decimal coste;
        private ObservableCollection<Negocio> listaNegocios;
        private ObservableCollection<Polimero> listaPolimeros;
        private ObservableCollection<Aplicacion> listaAplicaciones;
        private ObservableCollection<Moneda> listaDivisas;
        private Negocio selectedNegocio;
        private Polimero selectedPolimero;
        private Aplicacion selectedAplicacion;
        private Moneda selectedDivisa;
        private bool isRunning;
        private bool isEnabled;
        private ObservableCollection<string> listaArticulos = new ObservableCollection<string>();
        #endregion

        #region Properties
        public string Articulo
        {
            get { return this.articulo; }
            set { SetValue(ref this.articulo, value); }
        }

        public decimal Coste
        {
            get { return this.coste; }
            set { SetValue(ref this.coste, value); }
        }

        public ObservableCollection<Negocio> ListaNegocios
        {
            get { return this.listaNegocios; }
            set { SetValue(ref this.listaNegocios, value); }
        }

        public Negocio SelectedNegocio
        {
            get { return this.selectedNegocio; }
            set { SetValue(ref this.selectedNegocio, value); }
        }

        public ObservableCollection<Polimero> ListaPolimeros
        {
            get { return this.listaPolimeros; }
            set { SetValue(ref this.listaPolimeros, value); }
        }

        public Polimero SelectedPolimero
        {
            get { return this.selectedPolimero; }
            set { SetValue(ref this.selectedPolimero, value); }
        }

        public ObservableCollection<Aplicacion> ListaAplicaciones
        {
            get { return this.listaAplicaciones; }
            set { SetValue(ref this.listaAplicaciones, value); }
        }

        public Aplicacion SelectedAplicacion
        {
            get { return this.selectedAplicacion; }
            set { SetValue(ref this.selectedAplicacion, value); }
        }
        public ObservableCollection<Moneda> ListaDivisas
        {
            get { return this.listaDivisas; }
            set { SetValue(ref this.listaDivisas, value); }
        }

        public Moneda SelectedDivisa
        {
            get { return this.selectedDivisa; }
            set { SetValue(ref this.selectedDivisa, value); }
        }

        public bool IsRunning
        {
            get { return this.isRunning; }
            set { SetValue(ref this.isRunning, value); }
        }

        public bool IsEnabled
        {
            get { return this.isEnabled; }
            set { SetValue(ref this.isEnabled, value); }
        }
        public ObservableCollection<string> ListaArticulos
        {
            get { return this.listaArticulos; }
            set { SetValue(ref this.listaArticulos, value); }
        }
        #endregion

        #region Commands
        public ICommand CalcularCommand
        {
            get
            {                
                return new Command(async () =>
                {
                    await Calcular();
                });
            }
        }
        #endregion

        #region Constructor
        public CalculadoraViewModel()
        {
            if (Settings.IsLoggedIn == false)
            {
                App.Current.MainPage.Navigation.PushAsync(new LoginPage());
            }
                        
            this.CargarDatos();
        }
        #endregion
        #region Mehods
        private async Task Calcular()
        {
            Thread.CurrentThread.CurrentUICulture = (CultureInfo)Application.Current.Properties["currentLanguage"];

            try
            {
                this.IsRunning = true;
                UserDialogs.Instance.ShowLoading(Textos.Calculando);

                var apiAuxiliar = new ConexionAPI();
                var connection = await apiAuxiliar.CheckConnection();
                if (!connection.IsSuccess)
                {
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();
                    await App.Current.MainPage.DisplayAlert(Textos.Error, connection.Message, Textos.Aceptar);
                    return;
                }

                if (String.IsNullOrEmpty(this.Articulo) && this.Coste == 0)
                {
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();
                    await App.Current.MainPage.DisplayAlert(Textos.Ofertas, Textos.ErrorArticuloCoste, Textos.Aceptar);
                    return;
                }

                if (SelectedNegocio == null || SelectedNegocio.IdNegocio == 0)
                {
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();
                    await App.Current.MainPage.DisplayAlert(Textos.Ofertas, Textos.ErrorSeleccionNegocio, Textos.Aceptar);
                    return;
                }
                
                if (SelectedPolimero == null || SelectedPolimero.IdPolimero == 0)
                {
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();
                    await App.Current.MainPage.DisplayAlert(Textos.Ofertas, Textos.ErrorSeleccionPolimero, Textos.Aceptar);
                    return;
                }

                if (SelectedAplicacion == null || SelectedAplicacion.IdAplicacion == 0)
                {
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();
                    await App.Current.MainPage.DisplayAlert(Textos.Ofertas, Textos.ErrorSeleccionAplicacion, Textos.Aceptar);
                    return;
                }

                if (SelectedDivisa == null || String.IsNullOrEmpty(SelectedDivisa.Divisa))
                {
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();
                    await App.Current.MainPage.DisplayAlert(Textos.Ofertas, Textos.ErrorSeleccionDivisa, Textos.Aceptar);
                    return;
                }

                //Hago el calculo de la oferta
                var parametro = new ParametrosCalculoPrecio
                {
                    IdArticulo = this.Articulo,
                    Coste = this.Coste,
                    IdAplicacion = SelectedAplicacion.IdAplicacion,
                    IdPolimero = SelectedPolimero.IdPolimero,
                    IdNegocio = SelectedNegocio.IdNegocio,
                    Moneda = selectedDivisa.Divisa,
                    MonedaValorCambio = (decimal)SelectedDivisa.ValorCambio
                };

                var api = new CalculadoraAPI();

                var precio = await api.ObtenerPrecio(parametro);

                if (precio == null || precio.Count==0)
                {

                    await App.Current.MainPage.DisplayAlert(Textos.Ofertas, Textos.NoExisteArticulo, Textos.Aceptar);

                }
                else
                {

                    await Application.Current.MainPage.Navigation.PushAsync(new PreciosPage(precio, this.Articulo));

                }

                IsRunning = false;
                UserDialogs.Instance.HideLoading();
            }
            catch(Exception ex)
            {
                this.IsRunning = false;
                UserDialogs.Instance.HideLoading();
                await App.Current.MainPage.DisplayAlert(Textos.Ofertas, Textos.ErrorCalculandoPrecio, Textos.Aceptar);
            }
        }
        //Carrega les dades dels tickers
        public async void CargarDatos()
        {
            Thread.CurrentThread.CurrentUICulture = (CultureInfo)Application.Current.Properties["currentLanguage"];

            try
            {
                this.IsEnabled = false;
                this.IsRunning = true;
                UserDialogs.Instance.ShowLoading(Textos.Cargandodatos);

                await CargarNegocios();
                await CargarPolimeros();                
                await CargarDivisas();
                await CargarArticulos();

                IsEnabled = true;
                IsRunning = false;
                UserDialogs.Instance.HideLoading();

            }
            catch(Exception ex)
            {
                await App.Current.MainPage.DisplayAlert(Textos.Error, ex.Message, Textos.Aceptar);
                this.IsEnabled = true;
                this.IsRunning = false;
                UserDialogs.Instance.HideLoading();
            }
        }
        private async Task CargarNegocios()
        {
            
            var api = new CalculadoraAPI();
            var lista = await (api.ObtenerNegocios());

            this.ListaNegocios = new ObservableCollection<Negocio>(lista);
        }

        private async Task CargarPolimeros()
        {
            var api = new CalculadoraAPI();
            var lista = await api.ObtenerPolimeros();

            this.ListaPolimeros = new ObservableCollection<Polimero>(lista);
        }

        public async Task CargarAplicaciones(short idNegocio)
        {
            var api = new CalculadoraAPI();
            var lista = await api.ObtenerAplicaciones(idNegocio);

            this.ListaAplicaciones = new ObservableCollection<Aplicacion>(lista);
        }

        private async Task CargarDivisas()
        {
            var api = new CalculadoraAPI();
            var lista = await api.ObtenerDivisas();

            this.ListaDivisas = new ObservableCollection<Moneda>(lista);
                        
        }
        //Carrega les dades d'article per poder fer un autocomplete
        private async Task CargarArticulos()
        {
            var api = new MaestrosAPI();
            var lista = await api.ObtenerArticulos();

            this.ListaArticulos = new ObservableCollection<string>(lista);

        }
        #endregion
    }
}
