﻿using Acr.UserDialogs;
using IQAP.OfertasApp.Helper;
using IQAP.OfertasApp.Recursos;
using IQAP.OfertasApp.Views;
using IQAP.OfertasAPP.Negocio.API;
using IQAP.OfertasAPP.Negocio.Helpers;
using IQAP.OfertasAPP.Negocio.Model;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net.Mail;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using Xamarin.Forms;

namespace IQAP.OfertasApp.ViewModels
{
    public class LineasViewModel: BaseViewModel
    {
        #region Atributos
        private int idOferta;
        private List<LineaOferta> listaLineasOfertas;
        private bool isRunning;
        private bool isEnabled;
        private string email;
        #endregion

        #region Properties
        public List<LineaOferta> ListaLineasOfertas
        {
            get { return this.listaLineasOfertas; }
            set { SetValue(ref this.listaLineasOfertas, value); }
        }

        public bool IsRunning
        {
            get { return this.isRunning; }
            set { SetValue(ref this.isRunning, value); }
        }

        public bool IsEnabled
        {
            get { return this.isEnabled; }
            set { SetValue(ref this.isEnabled, value); }
        }
        public int IdOferta
        {
            get { return this.idOferta; }
            set { SetValue(ref this.idOferta, value); }
        }
        public string Email
        {
            get { return this.email; }
            set { SetValue(ref this.email, value); }
        }
        #endregion

        #region Commands
        public ICommand EnviarCommand
        {
            get
            {
                return new Command(async () =>
                {
                    await EnviarMail();
                });
            }
        }
        #endregion

        #region Constructor
        public LineasViewModel()
        {
            this.IsEnabled = true;
            this.IsRunning = false;

            if (Settings.IsLoggedIn == false)
            {
                App.Current.MainPage.Navigation.PushAsync(new LoginPage());
            }           
        }
        #endregion

        #region Methods
        //Obté el preu de les linies
        public async Task MostrarPrecio(int idLineaOferta, string article)
        {
            //Tradueix al idioma seleccionat els textos
            Thread.CurrentThread.CurrentUICulture = (CultureInfo)Application.Current.Properties["currentLanguage"];

            try
            {
                var apiAuxiliar = new ConexionAPI();
                var connection = await apiAuxiliar.CheckConnection();
                if (!connection.IsSuccess)
                {
                    await App.Current.MainPage.DisplayAlert(Textos.Error, connection.Message, Textos.Aceptar);
                    return;
                }

                this.IsRunning = true;
                UserDialogs.Instance.ShowLoading(Textos.Cargando);
                this.IsEnabled = false;
                var api = new OfertasAPI();

                var precio = await api.ObtenerPrecio(idLineaOferta);

                if (precio == null || precio.Count == 0)
                {

                    await App.Current.MainPage.DisplayAlert(Textos.Ofertas, Textos.ErrorIntroducirPrecios, Textos.Aceptar);

                }
                else
                {
                    //Te dades per tant passa a la pantalla per mostrar els preus
                    await App.Current.MainPage.Navigation.PushAsync(new PreciosPage(precio, article));

                }

                this.IsEnabled = true;
                UserDialogs.Instance.HideLoading();
                this.IsRunning = false;

            }
            catch (Exception ex)
            {
                this.IsEnabled = true;
                UserDialogs.Instance.HideLoading();
                this.IsRunning = false;
                await App.Current.MainPage.DisplayAlert(Textos.Error, ex.Message, Textos.Aceptar);
            }
        }
        //Mètode per enviar l'oferta per mail
        public async Task EnviarMail()
        {
            Thread.CurrentThread.CurrentUICulture = (CultureInfo)Application.Current.Properties["currentLanguage"];

            try
            {
                var apiAuxiliar = new ConexionAPI();
                var connection = await apiAuxiliar.CheckConnection();
                if (!connection.IsSuccess)
                {
                    await App.Current.MainPage.DisplayAlert(Textos.Error, connection.Message, Textos.Aceptar);
                    return;
                }

                this.IsRunning = true;
                UserDialogs.Instance.ShowLoading(Textos.EnviandoMail);
                this.IsEnabled = false;

                if (string.IsNullOrEmpty(this.Email))
                {
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();

                    await Application.Current.MainPage.DisplayAlert(Textos.Error, Textos.ErrorDireccionEmail, Textos.Aceptar);
                    return;
                }

                if (ValidaEmail(this.Email) == false)
                {
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();

                    await Application.Current.MainPage.DisplayAlert(Textos.Error, Textos.ErrorEmailCorrecto, Textos.Aceptar);
                    return;
                }

                //recupere els articules
                List<string> articulos = this.listaLineasOfertas.Select(l => l.Articulo).ToList();

                var api = new OfertasAPI();
                var res = await api.EnviarMail(idOferta, articulos, this.Email);

                this.IsRunning = false;
                UserDialogs.Instance.HideLoading();
                this.IsEnabled = true;

                if (res)
                {
                    await App.Current.MainPage.DisplayAlert(Textos.Ofertas, Textos.EmailEnviado, Textos.Aceptar);
                }
                else
                {
                    await App.Current.MainPage.DisplayAlert(Textos.Ofertas, Textos.EmailNoEnviado, Textos.Aceptar);
                }
            }
            catch(Exception ex)
            {
                this.IsEnabled = true;
                UserDialogs.Instance.HideLoading();
                this.IsRunning = false;
                await App.Current.MainPage.DisplayAlert(Textos.Error, ex.Message, Textos.Aceptar);
            }
        }
        //Valida que l'adreça de mail es correcta
        private bool ValidaEmail(string email)
        {
            try
            {
                MailAddress m = new MailAddress(email);

                return true;
            }
            catch (FormatException)
            {
                return false;
            }

        }
        #endregion
    }
}
