﻿using Acr.UserDialogs;
using IQAP.Ofertas.Services.Models;
using IQAP.OfertasApp.Helper;
using IQAP.OfertasApp.Recursos;
using IQAP.OfertasApp.Views;
using IQAP.OfertasAPP.Negocio.API;
using IQAP.OfertasAPP.Negocio.Helpers;
using IQAP.OfertasAPP.Negocio.Model;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using Xamarin.Forms;

namespace IQAP.OfertasApp.ViewModels
{
    public class ListadoOfertaViewModel: BaseViewModel
    {

        #region Atributos
        private List<Oferta> listaOfertas;
        private bool isRunning;
        private bool isEnabled;
        #endregion

        #region Properties
        public List<Oferta> ListaOfertas
        {
            get { return this.listaOfertas; }
            set { SetValue(ref this.listaOfertas, value); }
        }

        public bool IsRunning
        {
            get { return this.isRunning; }
            set { SetValue(ref this.isRunning, value); }
        }

        public bool IsEnabled
        {
            get { return this.isEnabled; }
            set { SetValue(ref this.isEnabled, value); }
        }
        #endregion
        
        #region Constructor
        public ListadoOfertaViewModel()
        {
            if (Settings.IsLoggedIn == false)
            {
                App.Current.MainPage.Navigation.PushAsync(new LoginPage());
            }

            this.IsRunning = false;
            this.IsEnabled = true;
        }
        #endregion

        #region Methods
        public async Task ObtenerLineas(int idOferta)
        {
            Thread.CurrentThread.CurrentUICulture = (CultureInfo)Application.Current.Properties["currentLanguage"];

            try
            {
                var apiAuxiliar = new ConexionAPI();
                var connection = await apiAuxiliar.CheckConnection();
                if (!connection.IsSuccess)
                {
                    await App.Current.MainPage.DisplayAlert(Textos.Error, connection.Message, Textos.Aceptar);
                    return;
                }

                this.IsRunning = true;
                UserDialogs.Instance.ShowLoading(Textos.Cargandodatos);
                this.IsEnabled = false;

                var api = new OfertasAPI();
                var lista = await api.ObtenerLineasOferta(idOferta);

                this.IsRunning = false;
                UserDialogs.Instance.HideLoading();
                this.IsEnabled = true;
                if (lista != null)
                {
                    await App.Current.MainPage.Navigation.PushAsync(new LineasPage(idOferta, lista));
                }
                else
                {
                    await App.Current.MainPage.DisplayAlert(Textos.Ofertas, Textos.NoRegistros, Textos.Aceptar);
                }
            }
            catch (Exception ex)
            {

                this.IsRunning = false;
                UserDialogs.Instance.HideLoading();
                await App.Current.MainPage.DisplayAlert(Textos.Error, ex.Message, Textos.Aceptar);

            }
        }
        #endregion
    }
}
