﻿using Acr.UserDialogs;
using IQAP.OfertasApp.Helper;
using IQAP.OfertasApp.Recursos;
using IQAP.OfertasApp.Views;
using IQAP.OfertasAPP.Negocio.API;
using IQAP.OfertasAPP.Negocio.Helpers;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using Xamarin.Forms;

namespace IQAP.OfertasApp.ViewModels
{
    public class LoginViewModel: BaseViewModel
    {
        #region Attributes
        private string usuario;
        private string password;
        private bool isRunning;        
        #endregion

        #region Properties
        public string Usuario
        {
            get { return this.usuario; }
            set
            {
                SetValue(ref this.usuario, value);                
            }
        }

        public string Password
        {
            get { return this.password; }
            set
            {
                SetValue(ref this.password, value);
                
            }
        }

        public bool IsRunning
        {
            get { return this.isRunning; }
            set { SetValue(ref this.isRunning, value); }
        }

        public bool IsRemembered
        {
            get;
            set;
        }
        #endregion

        #region Commands
        public ICommand LoginCommand
        {
            get
            {
                return new Command(async () =>
                {
                    await Login();
                });
            }
        }
        #endregion

        #region Constructor
        public LoginViewModel()
        {
            this.IsRemembered = Settings.IsRemember;
            this.IsRunning = false;

            if (this.IsRemembered)
            {
                this.Usuario = Settings.LastUser;
                this.Password = Settings.LastPassword;
            }
            
        }
        #endregion

        #region Methods
        private async Task Login()
        {
            Thread.CurrentThread.CurrentUICulture = (CultureInfo)Application.Current.Properties["currentLanguage"];

            try
            {
                this.IsRunning = true;
                
                UserDialogs.Instance.ShowLoading(Textos.Validando);

                var apiAuxiliar = new ConexionAPI();
                var connection = await apiAuxiliar.CheckConnection();
                if (!connection.IsSuccess)
                {
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();

                    await App.Current.MainPage.DisplayAlert(Textos.Error, connection.Message, Textos.Aceptar);                    
                    return;
                }                

                if (string.IsNullOrEmpty(this.Usuario))
                {
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();

                    await Application.Current.MainPage.DisplayAlert(Textos.Error, Textos.ErrorUsuarioVacio, Textos.Aceptar);
                    return;
                }

                if (string.IsNullOrEmpty(this.Password))
                {
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();

                    await Application.Current.MainPage.DisplayAlert(Textos.Error, Textos.ErrorPasswordVacio, Textos.Aceptar);
                    return;
                }

                var usuarioApi = new UsuarioAPI();

                var usuario = await usuarioApi.ObtenerUsuario(this.Usuario, this.Password);

                if (usuario == null)
                {
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();

                    await App.Current.MainPage.DisplayAlert(Textos.Error, Textos.UsuarioIncorrecto, Textos.Aceptar);
                    return;
                    
                }
                
                this.IsRunning = false;
                UserDialogs.Instance.HideLoading();

                Settings.IsLoggedIn = true;
                Settings.GeneralSettings = usuario;
                Settings.IsRemember = this.IsRemembered;

                if (this.IsRemembered)
                {
                    Settings.LastUser = this.Usuario;
                    Settings.LastPassword = this.Password;
                }
                else
                {
                    Settings.LastUser = String.Empty;
                    Settings.LastPassword = String.Empty;
                }

                Application.Current.MainPage = new NavigationPage(new MenuPage());
                                                
                this.Usuario = string.Empty;
                this.Password = string.Empty;

            }            
            catch (Exception ex)
            {
                this.IsRunning = false;
                UserDialogs.Instance.HideLoading();

                await App.Current.MainPage.DisplayAlert(Textos.Error, ex.Message, Textos.Aceptar);
            }
        }
        public async Task Traducir(string idioma)
        {
            try
            {
                Application.Current.Properties["currentLanguage"] = new CultureInfo(idioma);

                Application.Current.MainPage = new LoginPage();                

            }
            catch (Exception ex)
            {
                await App.Current.MainPage.DisplayAlert(Textos.Error, ex.Message, Textos.Aceptar);
            }

        }
        #endregion
    }
}
