﻿using Acr.UserDialogs;
using IQAP.Ofertas.Services.Models;
using IQAP.OfertasApp.Helper;
using IQAP.OfertasApp.Recursos;
using IQAP.OfertasApp.Views;
using IQAP.OfertasAPP.Negocio.API;
using IQAP.OfertasAPP.Negocio.Helpers;
using IQAP.OfertasAPP.Negocio.Model;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using Xamarin.Forms;

namespace IQAP.OfertasApp.ViewModels
{
    public class OfertasViewModel : BaseViewModel
    {
        #region Atributos
        private string cliente;
        private string articulo;
        private bool isRunning;
        private bool isEnabled;
        private ObservableCollection<string> listaArticulos = new ObservableCollection<string>();
        private ObservableCollection<string> listaClientes = new ObservableCollection<string>();
        #endregion

        #region Properties
        public string Cliente
        {
            get { return this.cliente; }
            set { SetValue(ref this.cliente, value); }
        }

        public string Articulo
        {
            get { return this.articulo; }
            set { SetValue(ref this.articulo, value); }
        }
        
        public bool IsRunning
        {
            get { return this.isRunning; }
            set { SetValue(ref this.isRunning, value); }
        }

        public bool IsEnabled
        {
            get { return this.isEnabled; }
            set { SetValue(ref this.isEnabled, value); }
        }
        public ObservableCollection<string> ListaArticulos
        {
            get { return this.listaArticulos; }
            set { SetValue(ref this.listaArticulos, value); }
        }
        public ObservableCollection<string> ListaClientes
        {
            get { return this.listaClientes; }
            set { SetValue(ref this.listaClientes, value); }
        }
        #endregion

        #region Commands
        public ICommand BuscarCommand
        {
            get
            {                
                return new Command(async () =>
                {
                    await Buscar();
                });
            }
        }
        #endregion

        #region Constructor
        public OfertasViewModel()
        {
            if (Settings.IsLoggedIn == false)
            {
                App.Current.MainPage.Navigation.PushAsync(new LoginPage());
            }

            this.CargarDatos();
        }
        #endregion

        #region Methods
        private async Task Buscar()
        {
            Thread.CurrentThread.CurrentUICulture = (CultureInfo)Application.Current.Properties["currentLanguage"];

            try
            {                

                var apiAuxiliar = new ConexionAPI();
                var connection = await apiAuxiliar.CheckConnection();
                if (!connection.IsSuccess)
                {
                    await App.Current.MainPage.DisplayAlert(Textos.Error, connection.Message, Textos.Aceptar);
                    return;
                }

                if (String.IsNullOrEmpty(this.Cliente) && String.IsNullOrEmpty(this.Articulo))
                {
                    await App.Current.MainPage.DisplayAlert(Textos.Error, Textos.ErrorClienteArticulo, Textos.Aceptar);
                }
                else
                {
                    this.IsRunning = true;
                    UserDialogs.Instance.ShowLoading(Textos.Buscando);

                    ParametrosOferta parametro = new ParametrosOferta();

                    parametro.Cliente = this.Cliente != null ? this.Cliente : String.Empty;
                    parametro.Articulo = this.Articulo != null ? this.Articulo : String.Empty;

                    var api = new OfertasAPI();
                    var lista = await api.ObtenerOfertas(parametro);
                    
                    this.IsRunning = false;
                    UserDialogs.Instance.HideLoading();

                    if (lista == null || lista.Count == 0)
                    {
                        await App.Current.MainPage.DisplayAlert(Textos.Ofertas, Textos.NoRegistros, Textos.Aceptar);
                    }
                    else
                    {
                        await App.Current.MainPage.Navigation.PushAsync(new ListadoOfertaPage(lista));
                    }
                }
            }
            catch (Exception ex)
            {

                this.IsRunning = false;
                UserDialogs.Instance.HideLoading();
                await App.Current.MainPage.DisplayAlert(Textos.Error, ex.Message, Textos.Aceptar);

            }
        }

        public async void CargarDatos()
        {
            Thread.CurrentThread.CurrentUICulture = (CultureInfo)Application.Current.Properties["currentLanguage"];

            try
            {
                this.IsEnabled = false;
                this.IsRunning = true;
                UserDialogs.Instance.ShowLoading(Textos.Cargandodatos);

                await CargarClientes();
                await CargarArticulos();


                IsEnabled = true;
                IsRunning = false;
                UserDialogs.Instance.HideLoading();

            }
            catch (Exception ex)
            {
                await App.Current.MainPage.DisplayAlert(Textos.Error, ex.Message, Textos.Aceptar);
                this.IsEnabled = true;
                this.IsRunning = false;
                UserDialogs.Instance.HideLoading();
            }
        }
        private async Task CargarClientes()
        {
            var api = new MaestrosAPI();
            var lista = await api.ObtenerClientes();

            this.ListaClientes = new ObservableCollection<string>(lista);
            
        }
        private async Task CargarArticulos()
        {
            var api = new MaestrosAPI();
            var lista = await api.ObtenerArticulos();

            this.ListaArticulos = new ObservableCollection<string>(lista);

        }
        #endregion
    }
}
